;===============================================================;
;                                                               ;
; Alien Breed 5                                                 ;
; Version 3.0.0                                                 ;
;                                                               ;
; By James Vernon                                               ;
; jamesv82@live.com.au                                          ;
;                                                               ;
; You are allowed to use parts of this code as long as you give ;
; credit where it is due :)                                     ;
;                                                               ;
; Based on the original Alien Breed games for Amiga by Team 17  ;
;                                                               ;
;===============================================================;

; debugging flags, should be commented out for normal play
;#define NO_CLIP
;#define INVINCIBLE
;#define LEVEL_SKIP
;#define ALL_ITEMS
;#define COOP_SYNC
;#define PONG_CHEAT
;#define DISABLE_TIPS
;#define FPS_TIMER_ON


#include "defines.asm"                          ; definitions

#ifdef TI83PLUS
.org    $4000
defpage(0,"AB5")
        jp      start
.db     0                                       ; pad out the header to a multiple of 3 bytes (for the branch table)
#endif

#ifdef TI84PLUSCE
.org    userMem-2
.db     tExtTok,tAsm84CeCmp
header:
        jp      start
        .db     1                               ; for cesium
        .db     16,16
        .db     $FF,$FF,$FF,$FF,$00,$00,$FF,$FF,$FF,$FF,$00,$00,$FF,$FF,$FF,$FF
        .db     $FF,$FF,$FF,$00,$00,$FF,$FF,$FF,$FF,$FF,$FF,$00,$00,$FF,$FF,$FF
        .db     $FF,$FF,$00,$00,$00,$00,$FF,$FF,$FF,$FF,$00,$00,$00,$00,$FF,$FF
        .db     $FF,$FF,$00,$00,$00,$FF,$FF,$FF,$FF,$FF,$FF,$00,$00,$00,$FF,$FF
        .db     $FF,$00,$00,$00,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$00,$00,$00,$FF
        .db     $FF,$00,$00,$00,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$00,$00,$00,$FF
        .db     $00,$00,$00,$00,$FF,$FF,$FF,$00,$00,$FF,$FF,$FF,$00,$00,$00,$00
        .db     $00,$00,$00,$FF,$FF,$FF,$00,$00,$00,$00,$FF,$FF,$FF,$00,$00,$00
        .db     $00,$00,$00,$00,$FF,$FF,$00,$00,$00,$00,$FF,$FF,$00,$00,$00,$00
        .db     $00,$00,$00,$00,$FF,$FF,$00,$00,$00,$00,$FF,$FF,$00,$00,$00,$00
        .db     $00,$00,$00,$00,$FF,$FF,$00,$00,$00,$00,$FF,$FF,$00,$00,$00,$00
        .db     $FF,$00,$00,$00,$00,$FF,$00,$00,$00,$00,$FF,$00,$00,$00,$00,$FF
        .db     $FF,$FF,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$FF,$FF
        .db     $FF,$FF,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$FF,$FF
        .db     $FF,$FF,$FF,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$FF,$FF,$FF
        .db     $FF,$FF,$FF,$FF,$00,$00,$00,$00,$00,$00,$00,$00,$FF,$FF,$FF,$FF
        .db     "Alien Breed v",VERSION_NUMBER,0
#include "fps.inc"
#endif

#include "branch.asm"                           ; branch table

;------------------------------------------------
; Start of program
;------------------------------------------------
start:
#ifdef TI83PLUS
        ; thanks to Michael V for this little nifty snippet :)
        in      a,($02)
        rla
        sbc     a,a
        out     ($20),a                         ; set cpu to 15Mhz (except on TI-83+BE, where it will simply set the link lines high)

        bcall(_delres)                          ; statVars invalid
        set     textwrite,(iy+sgrflags)         ; write text to gbuf

        ; clear system pointers after gbuf so we have a 1024 byte buffer for huffman decompression routine
        ld      hl,gbuf+768                     ; HL => system pointers
        ld      de,systemBackup                 ; DE => 256 bytes to store them at
        ld      bc,256
        ldir                                    ; save them
#endif
#ifdef TI84PLUSCE
        call    _runindicoff
        
        ; load saved passcodes
        call    loadPasscodeFile
        
        ; check for Ab5dat
        ld      hl,appvarDat
        MOVE9()
        call    _chkfindsym
        ld      hl,strDatFileMissing
        jp      c,fileError
        call    getDataPtr
        ld      hl,strDatFileVersion
        call    checkDataVersion
        ld      (pDatFile),de
        
        ; check for Ab5gfx1
        ld      hl,appvarGfx1
        MOVE9()
        call    _chkfindsym
        ld      hl,strGfxFile1Missing
        jp      c,fileError
        call    getDataPtr                      ; DE => start of gfx file (version number)
        ld      hl,strGfxFile1Version
        call    checkDataVersion
        ld      (__gfxHuffman1),de              ; save it for loadTileset
        
        ; check for Ab5gfx2
        ld      hl,appvarGfx2
        MOVE9()
        call    _chkfindsym
        ld      hl,strGfxFile2Missing
        jp      c,fileError
        call    getDataPtr
        ld      hl,strGfxFile2Version
        call    checkDataVersion
        ld      (__gfxHuffman2),de
        
        ; check for Ab5lvl
        ld      hl,appvarLvl
        MOVE9()
        call    _chkfindsym
        ld      hl,strLvlFileMissing
        jp      c,fileError
        call    getDataPtr
        ld      hl,strLvlFileVersion
        call    checkDataVersion
        ld      (pLvlFile),de
        
        ; prep LCD
        call    vramClear
        ld      hl,palGame
        ld      bc,PALGAME_SIZE
        call    loadPalette
        call    vram8bpp
        ld      hl,vbuf2
        ld      (pVbuf),hl
        ld      bc,VBUF_WIN_OFFSET
        add     hl,bc
        ld      (pVbufWin),hl

        res     apdAble,(iy+apdFlags)

        ; load character sprites
        call    loadChars

        ; default white text
        ld      a,COLOUR_WHITE
        ld      (__dcColour),a
#endif
        
        ld      hl,varsAppvar
        MOVE9()
        bcall(_chkfindsym)
        call    nc,loadState                    ; if vars file exists, try to load state

        ; check enough free RAM and create temp appvar
        ld      hl,MEM_NEEDED                   ; HL = required memory
        bcall(_enoughMem)                       ; check free memory
        jp      c,quit                          ; if not enough, quit
        call    checkTempFile                   ; call routine used at end of program to make sure the temp file doesn't already exist
        ld      hl,TEMP_DATA_SIZE               ; HL = size of temp appvar file
        bcall(_createappvar)                    ; create it
        inc     de
        inc     de                              ; DE => start of storage inside temp appvar file
        ld      (tempFile),de                   ; save pointer
#ifdef TI83PLUS
        ld      hl,LEVEL_DATA_SIZE
        add     hl,de
        ld      (pTiles),hl                     ; save pointer to where tile sprites will be stored
#endif

        call    loadAppvar
        push    af
        ld      hl,miscFlags
        bit     appvarFlag,(hl)
        jr      z,afterIntroCut
        
doIntro:
        ; "JamesV presents..."
        ld      hl,strJamesV                    ; HL => intro string
        call    displayPage                     ; type it out
        ld      b,160
        call    shortDelay                      ; wait B times or until [ALPHA] pressed
        jr      c,afterIntroCut                 ; if [ALPHA] was pressed, skip the cut scene

        ; show intro cut scene
#ifdef TI83PLUS
        ld      c,2
        ld      hl,intro
        call    doCutScene
        ld      a,$FF
        call    clearSplashAnim
#endif
#ifdef TI84PLUSCE
        ld      b,NUMEPISODES
        ld      hl,palIntro
        call    doCutScene
#endif

afterIntroCut:
        ; clear all cheat flags
        xor     a
        ld      hl,e1cheatFlags
        ld      (hl),a
        inc     hl
        ld      (hl),a
        inc     hl
        ld      (hl),a

        pop     af
        jp      z,mainMenu                      ; if appvar loaded, go to main menu
        ; otherwise, fall through to quit

quit:
        ld      hl,appQuit
        push    hl                              ; force program to ret to appQuit, so that it can close the app properly
checkTempFile:
        ld      hl,tempAppvar                   ; HL => temporary appvar file name
        MOVE9()                                 ; move it to OP1
        bcall(_chkfindsym)                      ; check if it exists
        bcallnc(_delvararc)                     ; if so, delete it
        ret                                     ; ret either to app loading routine or app quitting routing
appQuit:
#ifdef TI83PLUS
        ld      hl,systemBackup                 ; HL => backed up system pointers
        ld      de,gbuf+768                     ; DE => where to restore them to
        ld      bc,256
        ldir                                    ; restore them
        res     textwrite,(iy+sgrflags)         ; write text directly to lcd
#endif
        ld      hl,dataAppvar
        MOVE9()
        bcall(_chkfindsym)                      ; find the data appvar (which will be in ram if it's on the calc)
        bcallnc(_arc_unarc)                     ; archive the data appvar
        set     apdable,(iy+apdflags)           ; ensure that apd is enabled before returning to OS
        set     apdrunning,(iy+apdflags)
#ifdef TI83PLUS
        bjump(_JForceCmdNoChar)                 ; quit app and return to TI-OS
#endif
#ifdef TI84PLUSCE
        call    savePasscodeFile
        ld      hl,vram
        ld      (mpLcdBase),hl
        call    vram16bpp
        call    _clrlcdfull
        call    _homeup
        call    _drawstatusbar
        ei
        ret
#endif

newGame:
#ifdef TI84PLUSCE
        push    bc
        call    fadeOutClear
        pop     bc
#endif
        xor     a                               ; A = 0
        sbc     hl,hl                           ; HL = 0
        ld      (level),a
        ld      (prevLevel),a
        ld      (p1k),a                         ; keys = 0
        ld      (p2k),a
        ld      (credits),hl                    ; credits = 0
        ld      hl,e3CheatFlags
        bit     moneyCheat,(hl)
        jr      z,noMoneyCheat
        ld      hl,20000
        ld      (credits),hl
noMoneyCheat:
        inc     a                               ; A = 1
        ld      (weaponTable),a
        ld      (aflagLongPlay),a
        ld      a,b                             ; A = episode #
        ld      (episode),a
#ifdef TI83PLUS
        ld      h,0
        ld      l,a                             ; HL = A
        add     hl,hl
        ld      de,episodeCutTable
        add     hl,de                           ; HL => cut scene data for this episode
        LDHLIND()
        ld      a,2
        add     a,b
        ld      c,a                             ; C = offset to app page with data
        call    doCutScene
        xor     a                               ; A = 0 to clear lcd with clearSplashAnim
        call    clearSplashAnim
#endif
#ifdef TI84PLUSCE
        ld      e,b
        ld      d,3
        mlt     de
        ld      hl,cutscenePal
        add     hl,de
        ld      hl,(hl)
        call    doCutScene
#endif
        ; check if co-op game or not
        ld      hl,(pOptions)
        ld      a,(hl)
        ld      bc,0
        and     OPTION_PLAYERS
        jr      z,setGameType
        inc     b
        inc     c
setGameType:
        ld      a,b
        ld      (coop),a                        ; set/reset co-op flag
        ld      a,c
        ld      (p2p),a                         ; set/reset player 2 active flag
        ld      a,1
        ld      (p1p),a                         ; player 1 is always active at start of game
        ld      hl,miscFlags
        res     passcodeUsed,(hl)
        jr      startGame

newPCGame:
#ifdef TI84PLUSCE
        push    bc
        call    fadeOutClear
        pop     bc
#endif
        ld      hl,miscFlags
        set     passcodeUsed,(hl)
        xor     a
        ld      (aflagLongPlay),a               ; can't get a long play achievement when starting with a passcode
        ld      a,6
        ld      (p1k),a                         ; keys = 6 if you start with a passcode in a 1 player game
        ld      hl,coop
        bit     0,(hl)
        jr      z,startGame
        ; if a 2 player game, both players start with 3 keys
        srl     a
        ld      (p1k),a
        ld      (p2k),a
        ; fall through to startGame

startGame:
#ifdef TI83PLUS
        res     textInverse,(iy+textFlags)
#endif
#ifdef TI84PLUSCE
        call    vramClear
#endif
        ; check difficulty
        ld      hl,(pOptions)
        ld      a,(hl)
        ld      b,0
        and     OPTION_DIFFICULTY
        jr      z,setDifficulty
        inc     b
setDifficulty:
        ld      a,b
        ld      (difficulty),a
        ; check for all weapons cheat
        ld      hl,e2cheatFlags
        bit     weaponsCheat,(hl)
        jr      z,afterWeaponsCheat
        ld      a,(weaponTable)
        or      %00011111                       ; give player all weapons
        ld      (weaponTable),a
afterWeaponsCheat:
        xor     a                               ; A = 0
        sbc     hl,hl                           ; HL = 0
        ld      (p1d),a                         ; direction = up
        ld      (p2d),a
        ld      (p1s),hl                        ; score = 0
        ld      (p2s),hl
        ld      (bulletSlide),a
        ld      (routeFlags),a
        inc     a                               ; A = 1
        ld      (p1w),a
        ld      (p2w),a
        ld      a,(miscFlags)
        and     %00001000
        ld      (miscFlags),a
        ld      a,4
        ld      (p1l),a                         ; lives = 4
        ld      (p2l),a
        ld      a,MAX_HEALTH
        ld      (p1h),a                         ; health = MAX_HEALTH
        ld      (p2h),a
        ld      hl,2*256+255                    ; 2*unopened clips + 1*full clip
        ld      (p1a),hl
        ld      (p2a),hl

        jp      loadLevel                       ; load level

;------------------------------------------------
; levelFinished - do end of level stuff (or end of episode stuff)
;   input:  IX => next level number
;   output: none
;------------------------------------------------
levelFinishedCheat:
        pop     hl
levelFinished:
#ifdef TI83PLUS
        pop hl \ pop hl \ pop hl                ; clear bcall from stack
#endif
#ifdef TI84PLUSCE
        pop     hl
        push    ix
        call    fadeOutClear
        pop     ix
#endif
        push    ix
#ifdef TI83PLUS
        call    resetZOffset                    ; ensure LCD Z-offset is reset
        call    fadeOut                         ; fade out lcd
        bcall(_grbufclr)
        bcall(_grbufcpy)
        call    restoreContrast                 ; restore lcd contrast
#endif
        ; check for co-op finished a level achievement
        ld      a,(p1p)
        ld      hl,p2p
        add     a,(hl)
        cp      2
        jr      nz,afterCoopAchCheck            ; if there isn't 2 active players, no achievement
        ld      bc,ACH_COOPGAME_BIT*256+ACH_COOPGAME_ID
        ld      de,ACH_COOPGAME_OFFSET
        ld      hl,viewNewAchievementSaveScr
        bcall(ab5_setAchievement)
afterCoopAchCheck:
        ; check for 0 secs left on the countdown achievement
        ld      a,(levelFlags)
        bit     LFLAG_COUNTDOWNON,a
        jr      z,afterZeroSecsAch
        ld      a,(time)
        or      a
        jr      nz,afterZeroSecsAch
        ld      bc,ACH_0SECS_BIT*256+ACH_0SECS_ID
        ld      de,ACH_0SECS_OFFSET
        ld      hl,viewNewAchievement
        bcall(ab5_setAchievement)
afterZeroSecsAch:
        ; check for episode 1 / level 7 beat the timer achievement
        ld      a,(episode)
        or      a
        jr      nz,afterE1L7TimeAch
        ld      a,(level)
        cp      6
        jr      nz,afterE1L7TimeAch
        ld      a,(time)
        cp      30
        jr      c,afterE1L7TimeAch
        ld      bc,ACH_EP1L7TIME_BIT*256+ACH_EP1L7TIME_ID
        ld      de,ACH_EP1L7TIME_OFFSET
        ld      hl,viewNewAchievement
        bcall(ab5_setAchievement)
afterE1L7TimeAch:
        ; check for episode 2 / level 7 beat the timer achievement
        ld      a,(episode)
        dec     a
        jr      nz,afterE2L7TimeAch
        ld      a,(level)
        cp      6
        jr      nz,afterE2L7TimeAch
        ld      a,(time)
        cp      40
        jr      c,afterE2L7TimeAch
        ld      bc,ACH_EP2L7TIME_BIT*256+ACH_EP2L7TIME_ID
        ld      de,ACH_EP2L7TIME_OFFSET
        ld      hl,viewNewAchievement
        bcall(ab5_setAchievement)
afterE2L7TimeAch:
        ; check for episode 3 / main tower level 5 beat the timer achievement
        ld      a,(episode)
        cp      2
        jr      nz,afterE3TMA5TimeAch
        ld      a,(level)
        cp      29
        jr      nz,afterE3TMA5TimeAch
        ld      a,(time)
        cp      55
        jr      c,afterE3TMA5TimeAch
        ld      bc,ACH_EP3TMA5TIME_BIT*256+ACH_EP3TMA5TIME_ID
        ld      de,ACH_EP3TMA5TIME_OFFSET
        ld      hl,viewNewAchievement
        bcall(ab5_setAchievement)
afterE3TMA5TimeAch:
        ; check for episode 1 / level 11 pacifist achievement
        ld      a,(episode)
        or      a
        jr      nz,afterEp1Peace
        ld      a,(level)
        cp      10
        jr      nz,afterEp1Peace
        ld      a,(aflagAliens)
        or      a
        jr      z,afterEp1Peace
        ld      bc,ACH_EP1PEACE_BIT*256+ACH_EP1PEACE_ID
        ld      de,ACH_EP1PEACE_OFFSET
        ld      hl,viewNewAchievement
        bcall(ab5_setAchievement)
afterEp1Peace:
        ; check for episode 2 / level 4 pickups achievement
        ld      a,(episode)
        dec     a
        jr      nz,afterE2L4Pickups
        ld      a,(level)
        cp      3
        jr      nz,afterE2L4Pickups
        ld      a,(numPickups)
        cp      NUM_PUPS_E2L4
        jr      c,afterE2L4Pickups
        ld      bc,ACH_EP2L4PUPS_BIT*256+ACH_EP2L4PUPS_ID
        ld      de,ACH_EP2L4PUPS_OFFSET
        ld      hl,viewNewAchievement
        bcall(ab5_setAchievement)
afterE2L4Pickups:
        ; check for episode 2 / level 5 doors achievement
        ld      a,(episode)
        dec     a
        jr      nz,afterE2L5Doors
        ld      a,(level)
        cp      4
        jr      nz,afterE2L5Doors
        ld      a,(numDoors)
        cp      NUM_DOORS_E2L5
        jr      c,afterE2L5Doors
        ld      bc,ACH_EP2L5DOORS_BIT*256+ACH_EP2L5DOORS_ID
        ld      de,ACH_EP2L5DOORS_OFFSET
        ld      hl,viewNewAchievement
        bcall(ab5_setAchievement)
afterE2L5Doors:
        ld      hl,SCORE_LEVEL
        ld      ix,p1x
        push    hl
        bcall(ab5_addScore)                     ; add score for finishing a level
        pop     hl
        ld      ix,p2x
        bcall(ab5_addScore)
        ld      a,(level)
        ld      (prevLevel),a                   ; save finished level #
#ifdef TI83PLUS
        ld      hl,(episode)
        ld      h,0
#endif
#ifdef TI84PLUSCE
        ld      a,(episode)
        LDHLA()
#endif
        ld      de,numlevelsTable
        add     hl,de                           ; HL => # levels in current episode
        pop     ix
        ld      a,(ix)                          ; A = next level
        ld      (level),a                       ; save it
        cp      (hl)                            ; check to see if the episode is finished
        jp      z,episodeFinished               ; if so, go deal with that

        ; generate & draw the descending elevator screen
#ifdef TI83PLUS
        ld      b,64                            ; B = 64 gbuf lines to generate
        ld      de,gbuf                         ; DE => start of gbuf
drawElevatorShaft:
        push    bc                              ; save line counter
        ld      b,12                            ; B = 12 bytes per gbuf line
        ld      hl,elevatorScrMasks             ; HL => AND & OR data to use for the line
drawEShaftLine:
        push    bc                              ; save byte counter
        ld      b,5
        bcall(ab5_getRandom)                    ; A = 0-4
        ld      c,a
        ld      b,0                             ; BC = random #
        push    hl                              ; save mask data ptr
        ld      hl,elevatorScrRandoms
        add     hl,bc
        ld      a,(hl)                          ; A = random byte to display
        pop     hl                              ; restore mask data ptr
        and     (hl)                            ; apply the AND mask
        inc     hl
        or      (hl)                            ; apply the OR mask
        inc     hl                              ; HL => next pair of mask byte data
        ld      (de),a                          ; write byte to gbuf
        inc     de                              ; DE => next byte of gbuf
        pop     bc
        djnz    drawEShaftLine
        pop     bc
        djnz    drawElevatorShaft
#endif

        ; do the animation part
#ifdef TI83PLUS
        ld      b,88                            ; B = 88 frames
elevatorAnimLoop:
        push    bc                              ; save frame counter
        ld      a,b
        sub     64
        neg                                     ; A = y coord for this frame
        bcall(ab5_saveVideo)                    ; save background
        ld      b,4                             ; B = 4 sprites to draw
        ld      c,a                             ; C = y coord for this frame
        ld      a,(p1p)
        ld      hl,p2p
        add     a,(hl)                          ; A = number of active players
        dec     a
        ld      de,sprElevator1P                ; DE => start of sprites for 1 player
        jr      z,elevatorSprReady              ; if 1 player, good to go
        ld      de,sprElevator2P                ; otherwise use sprites for 2 players
elevatorSprReady:
        ld      hl,elevatorSprOffsets           ; HL => coord offset data
        call    elevatorAnimDraw
        ld      de,elevatorAnimDrawingDone
        push    de                              ; where to ret to after falling through to elevatorAnimDraw
        ld      b,2
        ld      de,sprElevatorBL1
        bit     1,c
        jr      z,elevatorAnimDraw              ; if an even frame, draw platform with jets
        ld      de,sprElevatorBL2               ; otherwise, draw platform without jets
elevatorAnimDraw:
        push    bc                              ; save sprite counter & y coord
        ld      b,(hl)                          ; B = x coord
        inc     hl
        ld      a,(hl)                          ; A = y offset
        inc     hl                              ; HL => offset data for next sprite
        add     a,c
        ld      c,a                             ; C = y coord
        ex      de,hl                           ; DE => offset data for next sprite; HL => sprite to draw
        push    de
        push    hl
        push    bc                              ; save x,y
        call    spriteToRam                     ; load sprite to ram
        ex      de,hl                           ; HL => sprite in ram
        pop     bc                              ; restore x,y
        bcall(ab5_putClippedSprite)             ; draw sprite
        pop     hl
        ld      bc,8
        add     hl,bc                           ; HL => next sprite to draw
        pop     de
        ex      de,hl                           ; DE => next sprite; HL => next offset data
        pop     bc                              ; restore sprite counter & y coord
        djnz    elevatorAnimDraw
        ret
elevatorAnimDrawingDone:
        bcall(_grbufcpy)
        bcall(ab5_restoreVideo)                 ; restore background without sprites
        pop     bc                              ; restore frame counter
        dec b                                   ; double the frame rate
        djnz    elevatorAnimLoop
#endif

        ld      hl,miscFlags
        res     passcodeUsed,(hl)
        jp      loadLevel                       ; start at next level

episodeFinished:
        call    doEpilogue
        call    wait2ndEnter
#ifdef TI84PLUSCE
        call    fadeOutClear
#endif

        ; set achievements
        ld      a,(episode)
        or      a
        jp      z,ep1Finished
        dec     a
        jp      z,ep2Finished
        ; if code reaches here, episode 3 was finished
        ; set standard finished episode 3 achievement
        ld      bc,ACH_EP3_BIT*256+ACH_EP3_ID
        ld      de,ACH_EP3_OFFSET
        ld      hl,viewNewAchievement
        bcall(ab5_setAchievement)
        ; check if should set long play episode 3 achievement
        ld      a,(aflagLongPlay)
        or      a
        jp      z,gameOver
        ld      bc,ACH_EP3LP_BIT*256+ACH_EP3LP_ID
        ld      de,ACH_EP3LP_OFFSET
        ld      hl,viewNewAchievement
        bcall(ab5_setAchievement)
        ; flag route taken, and check if all routes have been completed
        ld      hl,ep3RouteTotals
        ld      b,7
        ld      a,(routeFlags)
ep3CheckRoute:
        cp      (hl)
        jr      z,ep3RouteFound
        inc     hl
        djnz    ep3CheckRoute
        jr      gameOver
ep3RouteFound:
        ld      a,%10000000
ep3RouteShift:
        rrca
        djnz    ep3RouteShift
        ld      hl,(pAchVars)
        ld      de,ACH_COUNTER_SIZE
        add     hl,de
        or      (hl)
        ld      (hl),a
        cp      %01111111
        jr      nz,gameOver
        ld      bc,ACH_EP3PATHS_BIT*256+ACH_EP3PATHS_ID
        ld      de,ACH_EP3PATHS_OFFSET
        ld      hl,viewNewAchievement
        bcall(ab5_setAchievement)
        jr      gameOver
ep2Finished:
        ; set standard finished episode 2 achievement
        ld      bc,ACH_EP2_BIT*256+ACH_EP2_ID
        ld      de,ACH_EP2_OFFSET
        ld      hl,viewNewAchievement
        bcall(ab5_setAchievement)
        ; check if should set long play episode 2 achievement
        ld      a,(aflagLongPlay)
        or      a
        jr      z,gameOver
        ld      bc,ACH_EP2LP_BIT*256+ACH_EP2LP_ID
        ld      de,ACH_EP2LP_OFFSET
        ld      hl,viewNewAchievement
        bcall(ab5_setAchievement)
        jr      gameOver
ep1Finished:
        ; set standard finished episode 1 achievement
        ld      bc,ACH_EP1_BIT*256+ACH_EP1_ID
        ld      de,ACH_EP1_OFFSET
        ld      hl,viewNewAchievement
        bcall(ab5_setAchievement)
        ; check if should set long play episode 1 achievement
        ld      a,(aflagLongPlay)
        or      a
        jr      z,gameOver
        ld      bc,ACH_EP1LP_BIT*256+ACH_EP1LP_ID
        ld      de,ACH_EP1LP_OFFSET
        ld      hl,viewNewAchievement
        bcall(ab5_setAchievement)
        jr      gameOver

pdeadGameOver:
#ifdef TI83PLUS
        pop hl \ pop hl \ pop hl                ; clear bcall from stack
#endif
#ifdef TI84PLUSCE
        pop     hl
#endif

gameOver:
#ifdef TI84PLUSCE
        call    loadDefaultPalette
#endif
#ifdef TI83PLUS
        call    resetZOffset                    ; ensure LCD Z-offset is reset
        ld      a,$FF
        call    clearSplashAnim                 ; fill lcd with black
        bcall(_grbufclr)
        ld      hl,strGameOver
        ld      de,28*256+28
        call    vPuts                           ; "GAME OVER"
        call    invertGbuf
        bcall(_grbufcpy)
#endif
#ifdef TI84PLUSCE
        call    vbufClear
        ld      ix,strGameOver
        ld      de,92
        ld      l,112
        call    printString
        call    vramFlip
#endif
        call    wait2ndEnter
        
        ; check players for high scores
        call    iniMenu

        ld      ix,p1x
        ld      hl,strNewHighP1
        call    checkHighScore
        push    af                              ; save A (flag for whether or not player 1 got a high score)
        ld      ix,p2x
        ld      hl,strNewHighP2
        call    checkHighScore
        pop     bc
        add     a,b
        or      a                               ; did one or both players get a high score?
        jp      z,doMenuMain
        jp      doMenuViewHighScores

;------------------------------------------------
; page 0 includes
;------------------------------------------------
#include "appvar.asm"                           ; appvar routines
#include "event0.asm"                           ; event routines for app page 0
#include "high.asm"                             ; high score table routines
#include "huffextr.asm"                         ; huffman extraction routine
#include "intex.asm"                            ; intex routines
#include "level0.asm"                           ; level routines for app page 0
#include "menu.asm"                             ; menu routines
#include "misc0.asm"                            ; miscellaneous routines for app page 0
#include "passcode.asm"                         ; passcode routines
#include "pong.asm"                             ; space pong routines
#include "state.asm"                            ; save/load state routines
#include "tips.asm"                             ; in-game tips routines
#ifdef TI83PLUS
#include "cutscene.asm"                         ; cut scene routines
#include "info.asm"                             ; info box routines    
#include "pause.asm"                            ; pause routines
#include "text.asm"                             ; text routines
#include "vfx.asm"                              ; visual effect routines
#endif

#include "data0.asm"                            ; data for app page 0
e3egg:  #import "ep3\easter.bin"


;------------------------------------------------
; page 1 (main game play routines and data)
;------------------------------------------------
#ifdef TI83PLUS
defpage(1)                                      ; app page 1 starts here
#endif
#include "anim.asm"                             ; animation routines
#include "boss.asm"                             ; boss routines
#include "collide.asm"                          ; collision detection routines
#include "enemy.asm"                            ; enemy routines
#include "event1.asm"                           ; event routines for app page 1
#include "keys.asm"                             ; in-game key press routines
#include "level1.asm"                           ; level routines for app page 1
#include "maingame.asm"                         ; main game loop
#include "misc1.asm"                            ; miscellaneous routines for app page 1
#include "object.asm"                           ; object routines
#include "player.asm"                           ; player routines
#include "shoot.asm"                            ; shooting routines
#ifdef TI83PLUS
#include "gfx.asm"                              ; graphics routines (83+ only)
#include "scroll.asm"                           ; screen scrolling routines
#endif

#include "data1.asm"                            ; data for app page 1

#ifdef TI83PLUS
;------------------------------------------------
; page 2 (episode 1 compressed level data, end of episode splash data, game intro)
;------------------------------------------------
defpage(2)
e1l01:  #import "ep1\level01.huf"
e1l02:  #import "ep1\level02.huf"
e1l03:  #import "ep1\level03.huf"
e1l04:  #import "ep1\level04.huf"
e1l05:  #import "ep1\level05.huf"
e1l06:  #import "ep1\level06.huf"
e1l07:  #import "ep1\level07.huf"
e1l08:  #import "ep1\level08.huf"
e1l09:  #import "ep1\level09.huf"
e1l10:  #import "ep1\level10.huf"
e1l11:  #import "ep1\level11.huf"
e1l12:  #import "ep1\level12.huf"
e1ep:   #import "ep1\epilogue.huf"
splash: #import "gfx\splash.huf"                ; splash screen used in epilogues
intro:  #import "cut\intro.huf"                 ; game intro cut scene
ep1in:  #import "cut\ep1.huf"                   ; episode 1 cut scene

;------------------------------------------------
; page 3 (episode 2 compressed level data, tileset data)
;------------------------------------------------
defpage(3)
e2l01:  #import "ep2\level01.huf"
e2l02:  #import "ep2\level02.huf"
e2l03:  #import "ep2\level03.huf"
e2l04:  #import "ep2\level04.huf"
e2l05:  #import "ep2\level05.huf"
e2l06:  #import "ep2\level06.huf"
e2l07:  #import "ep2\level07.huf"
e2l08:  #import "ep2\level08.huf"
e2l09:  #import "ep2\level09.huf"
e2l10:  #import "ep2\level10.huf"
e2l11:  #import "ep2\level11.huf"
e2l12:  #import "ep2\level12.huf"
e2ep:   #import "ep2\epilogue.huf"
ep2in:  #import "cut\ep2.huf"                   ; episode 2 cut scene

;------------------------------------------------
; page 4 (episode 3 compressed level data)
;------------------------------------------------
defpage(4)
e3l00:  #import "ep3\level00.huf"
e3l01:  #import "ep3\level01.huf"
e3l02:  #import "ep3\level02.huf"
e3l03:  #import "ep3\level03.huf"
e3l04:  #import "ep3\level04.huf"
e3l05:  #import "ep3\level05.huf"
e3l06:  #import "ep3\level06.huf"
e3l07:  #import "ep3\level07.huf"
e3l08:  #import "ep3\level08.huf"
e3l09:  #import "ep3\level09.huf"
e3l10:  #import "ep3\level10.huf"
e3l11:  #import "ep3\level11.huf"
e3ep:   #import "ep3\epilogue.huf"
ep3in:  #import "cut\ep3.huf"                   ; episode 3 cut scene

;------------------------------------------------
; page 5 (episode 3 compressed level data)
;------------------------------------------------
defpage(5)
e3l12:  #import "ep3\level12.huf"
e3l13:  #import "ep3\level13.huf"
e3l14:  #import "ep3\level14.huf"
e3l15:  #import "ep3\level15.huf"
e3l16:  #import "ep3\level16.huf"
e3l17:  #import "ep3\level17.huf"
e3l18:  #import "ep3\level18.huf"
e3l19:  #import "ep3\level19.huf"
e3l20:  #import "ep3\level20.huf"
e3l21:  #import "ep3\level21.huf"
e3l22:  #import "ep3\level22.huf"
e3l23:  #import "ep3\level23.huf"
e3l24:  #import "ep3\level24.huf"
e3l25:  #import "ep3\level25.huf"
e3l26:  #import "ep3\level26.huf"
e3l27:  #import "ep3\level27.huf"
e3l28:  #import "ep3\level28.huf"
e3l29:  #import "ep3\level29.huf"

;------------------------------------------------
; page 6 (achievements)
;------------------------------------------------
defpage(6)
#include "achieve.asm"                          ; achievement routines    
#include "misc6.asm"                            ; miscellaneous routines for app page 6
#include "data6.asm"

validate()
#endif

#ifdef TI84PLUSCE
#include "achieve.asm"                          ; achievement routines    
#include "cutscenece.asm"
#include "gfxce.asm"
#include "hudce.asm"
#include "infoce.asm"
#include "levelce.asm"
#include "miscce.asm"
#include "pausece.asm"
#include "scrollce.asm"
#include "textce.asm"
#include "vfxce.asm"
#include "datace.asm"
#endif

.end
