;===============================================================;
;                                                               ;
; Alien Breed 5                                                 ;
; Boss routines                                                 ;
;                                                               ;
;===============================================================;

;------------------------------------------------
; iniBoss - initialise boss mode
;   input:  IX => data of player that triggered the boss
;           HL => 3 bytes of boss data {type/level, x, y)
;   output: none
;------------------------------------------------
iniBoss:
        push    hl
        ; if a 2P game, teleport the other player to the player that triggered the boss
        ld      a,(p1p)
        ld      hl,p2p
        add     a,(hl)
        cp      2                               ; 2 active players?
        jr      nz,iniBossAfterPort             ; if not, no porting to do
        di                                      ; we're going to use IY for a sec
        push    iy
        push    ix
        pop     hl
        ld      de,p1x
        CPHLDE()                                ; check to see if IX is pointing to player 1 data
        ld      iy,p2x                          ; IY => destination for player coord copying
        jr      z,iniBossDoPort                 ; if IX is pointing to player 1
        ld      iy,p1x                          ; otherwise player 1 is the destination (player 2 is the source)
iniBossDoPort:
        ; copy the x & y coords and the x & y screen coords
        ld      a,(ix+PX)
        ld      (iy+PX),a
        ld      a,(ix+PX+1)
        ld      (iy+PX+1),a
        ld      a,(ix+PY)
        ld      (iy+PY),a
        ld      a,(ix+PY+1)
        ld      (iy+PY+1),a
        ld      a,(ix+PSX)
        ld      (iy+PSX),a
        ld      a,(ix+PSY)
        ld      (iy+PSY),a
        pop     iy
        ei                                      ; safe for interrupts again as IY has been restored
iniBossAfterPort:
        call    enemyChooseTarget
        ld      (bossT),a                       ; set boss target
        ld      a,1
        ld      (aflagHookCrook),a              ; set flag for achievement, this will be reset if any weapon other than the machine gun is fired during the boss fight
        ld      (aflagBossCore),a               ; set flag to say that player has not died yet during the boss fight
        pop     ix                              ; IX => 3 bytes of boss data {type/level, x, y)
        ld      a,(ix)
        and     %00111111                       ; A = boss type
#ifdef TI83PLUS
        ld      l,a
        ld      h,0
        add     hl,hl
        ld      de,bossSpriteTable
        add     hl,de
        LDHLIND()                               ; HL => 1st sprite set for boss
        ld      (bossSprPtr),hl                 ; save ptr
#endif
#ifdef TI84PLUSCE
        push    ix
        ld      (bossType),a
        call    loadBossSprites
        pop     ix
#endif
        ld      a,(ix)                          ; A = boss data
        rlca
        rlca
        and     %00000011                       ; A = boss level
#ifdef TI83PLUS
        add     a,a                             ; *2
        ld      e,a
        ld      d,0
#endif
#ifdef TI84PLUSCE
        ld      e,a
        ld      d,3
        mlt     de
#endif
        ld      hl,bossLevelTable
        add     hl,de
        LDHLIND()                               ; HL = boss ini health
        ld      (bossHealth),hl                 ; save it
        xor     a
        ld      (bossSprCnt),a                  ; reset sprite animation counter
        ld      (bossState),a                   ; reset boss state
        ld      (bossFlash),a                   ; clear flash flag
        inc     a
        ld      (state),a                       ; set game state to 1 (boss mode)
        ld      l,(ix+1)
#ifdef TI83PLUS
        ld      h,0
#endif
        HLTIMES8()
        ld      (bossX),hl
        ld      l,(ix+2)
#ifdef TI83PLUS
        ld      h,0
#endif
        HLTIMES8()
        ld      (bossY),hl
        jp      despawnEnemies                  ; despawn enemies, and we're done prepping for the boss fight

;------------------------------------------------
; bossDead - kill the boss and return to normal game state
;   input:  none
;   output: none
;------------------------------------------------
bossDead:
        pop     hl                              ; clearing stack from collision detection routine
        pop     iy                              ; restore system flag ptr
        ei                                      ; and enable interrupts again
        call    trgBossKilled
        ; create explosion animations
        ld      b,7                             ; B = 7 animations
        ld      hl,bossExplosions               ; HL => offset data
bossDeadAnims:
        push    bc
#ifdef TI83PLUS
        ld      e,(hl)
        inc     hl
        ld      d,(hl)                          ; DE = x offset
        inc     hl
        ld      c,(hl)
        inc     hl
        ld      b,(hl)                          ; BC = y offset
        inc     hl                              ; HL => next set of offsets
#endif
#ifdef TI84PLUSCE
        call    _lddehl
        push    de
        call    _lddehl
        push de \ pop bc
        pop     de
#endif
        push    hl
        ld      hl,(bossX)
        add     hl,de
        ex      de,hl                           ; DE = x pos for explosion
        ld      hl,(bossY)
        add     hl,bc                           ; HL = y pos
        ld      bc,0*256+ANIM_EXPLOSION         ; B = animation type; C = counter ini
        call    newAnim
        pop     hl
        pop     bc
        djnz    bossDeadAnims
        ld      ix,p1x
        ld      hl,SCORE_BOSS
        push    hl
        call    addScore                        ; add score for killing boss
        pop     hl
        ld      ix,p2x
        call    addScore
        xor     a
        ld      (state),a                       ; reset game state to normal
        ld      a,(aflagHookCrook)
        or      a                               ; check if achievement should be given
        jr      z,afterHookCrook                ; if not, skip it
        ld      c,ACH_HOOKCROOK_ID
        ld      b,ACH_HOOKCROOK_BIT
        ld      de,ACH_HOOKCROOK_OFFSET
        ld      hl,viewNewAchievementSaveScr
        bcall(ab5_setAchievement)
afterHookCrook:
        ; check if boss was killed without any player deaths
        ld      a,(aflagBossCore)
        or      a
        jr      z,afterBossCore
        ; check if "OFFICER RIPLEY" achievement should be given
        ld      a,(episode)
        or      a
        jr      nz,afterRipley
        ld      a,(level)
        cp      11
        jr      nz,afterRipley
        ld      c,ACH_EP1BOSSCORE_ID
        ld      b,ACH_EP1BOSSCORE_BIT
        ld      de,ACH_EP1BOSSCORE_OFFSET
        ld      hl,viewNewAchievementSaveScr
        bcall(ab5_setAchievement)
afterRipley:
        ; inc boss hardcore kill achievements
        ld      bc,ACH_BOSSCORE1_BIT*256+ACH_BOSSCORE1_ID
        ld      de,ACH_BOSSCORE1_OFFSET
        ld      hl,1
        bcall(ab5_incAchievement)
        ld      bc,ACH_BOSSCORE2_BIT*256+ACH_BOSSCORE2_ID
        ld      de,ACH_BOSSCORE2_OFFSET
        ld      hl,1
        bcall(ab5_incAchievement)
afterBossCore:
        ; inc boss kill achievements
        ld      bc,ACH_BOSS1_BIT*256+ACH_BOSS1_ID
        ld      de,ACH_BOSS1_OFFSET
        ld      hl,1
        bcall(ab5_incAchievement)
        ld      bc,ACH_BOSS2_BIT*256+ACH_BOSS2_ID
        ld      de,ACH_BOSS2_OFFSET
        ld      hl,1
        bcall(ab5_incAchievement)
        ld      bc,ACH_BOSS3_BIT*256+ACH_BOSS3_ID
        ld      de,ACH_BOSS3_OFFSET
        ld      hl,1
        bcall(ab5_incAchievement)
        ret

;------------------------------------------------
; drawBoss - draw boss
;   input:  none
;   output: none
;------------------------------------------------
drawBoss:
        ld      hl,bossShot
        ld      de,bossFlash
        bit     0,(hl)                          ; was boss shot this frame?
        jr      z,clearBossFlash                ; if not clear the flash flag
        ld      a,(de)
        xor     $01                             ; toggle the flash flag (don't flash 2 frames in a row)
        ld      (de),a
        jr      drawBossAfterFlag
clearBossFlash:
        xor     a
        ld      (de),a
drawBossAfterFlag:
        ld      a,(bossSprCnt)
        sra     a
        sra     a                               ; /4 (animate boss every 4th frame)
        and     %00000011                       ; mask as boss has 4 possible sprites to display
#ifdef TI83PLUS
        add     a,a                             ; *2
        add     a,a                             ; *4
        add     a,a                             ; *8
        add     a,a                             ; *16
        add     a,a                             ; *32
        ld      e,a
        ld      d,0                             ; DE = *32
        add     a,a                             ; *64
        ld      l,a
        ld      h,d                             ; HL = *64 (remembering D was 0, shorter way of zeroing H)
        add     hl,de                           ; HL = *96, offset to sprite set
        ld      de,(bossSprPtr)
#endif
#ifdef TI84PLUSCE
        ld      l,a
        ld      h,192
        mlt     hl
        add     hl,hl
        add     hl,hl
        add     hl,hl                           ; *1536 (6 x 256 bytes per sprite)
        ld      de,sprBoss
#endif
        add     hl,de                           ; HL => sprite set
        ex      de,hl                           ; DE => sprite set
        ld      hl,(bossX)
        ld      bc,(screenX)
        or      a
        sbc     hl,bc                           ; L = x pos on screen (H is irrelevant)
        ld      a,l
        ld      (bossScrX),a                    ; save it
        ld      hl,(bossY)
        ld      bc,(screenY)
        or      a
        sbc     hl,bc                           ; L = y pos on screen (H is irrelevant)
        ld      a,l
        ld      (bossScrY),a                    ; save it
        ld      b,6                             ; B = number of sprites to draw
        ld      hl,bossDrawTable                ; HL = coords offset data
drawBossLoop:
        push    bc                              ; save loop counter
        push    de                              ; save sprite ptr
        ld      a,(bossScrX)
        add     a,(hl)
        ld      b,a                             ; B = x coord to display this sprite at
        inc     hl                              ; HL => y offset
        ld      a,(bossScrY)
        add     a,(hl)
        ld      c,a                             ; C = y coord to display this sprite at
        inc     hl                              ; HL => x offset for next sprite
        push    hl                              ; save offset data ptr
        ex      de,hl                           ; HL => sprite
        ld      a,(bossFlash)
        or      a
        jr      nz,drawBossFlash
#ifdef TI83PLUS
        call    putClippedMaskedSprite          ; draw sprite
#endif
#ifdef TI84PLUSCE
        push hl \ pop ix
        ld      e,b
        ld      d,c
        call    drawClippedTransSprite
#endif
        jr      bossSpriteDrawn
drawBossFlash:
#ifdef TI83PLUS
        ld      de,8
        add     hl,de                           ; HL => sprite mask (draw the actual mask to make the flash effect)
        call    putClippedSprite
#endif
#ifdef TI84PLUSCE
        push hl \ pop ix
        ld      e,b
        ld      d,c
        call    drawClippedTransSpriteWhite
#endif
bossSpriteDrawn:
        pop     de                              ; DE => x offset for next sprite
        pop     hl                              ; HL => sprite we just drew
#ifdef TI83PLUS
        ld      bc,16
#endif
#ifdef TI84PLUSCE
        ld      bc,256
#endif
        add     hl,bc                           ; HL => next sprite
        ex      de,hl                           ; DE => next sprite; HL => x offset for next sprite
        pop     bc                              ; restore loop counter
        djnz    drawBossLoop
        ret

;-----------------------------------------------
; moveBoss - move the boss
;   input:  none
;   output: none
;-----------------------------------------------
moveBoss:
        xor     a
        ld      (bossMoved),a                   ; clear flag
        ; check the boss state
        ld      a,(bossState)
        or      a
        jr      z,bossChase                     ; chase the player
        dec     a
        jp      z,bossLurk                      ; just lurkin'
bossRetreat:
        ld      hl,(bossY)
        dec     hl                              ; HL = bossY-1
        call    bossCheckVert
        jr      nc,bossRetreatDone
        ld      (bossY),hl                      ; save new y pos
        call    bossDidMove
        jp      moveBossDone
bossRetreatDone:
        xor     a
        ld      (bossState),a                   ; boss to start chasing next frame
        call    enemyChooseTarget
        ld      (bossT),a                       ; set boss target
        ret

bossChase:
        ; point IX to the correct target
        ld      a,(bossT)
        ld      ix,p1x
        or      a
        jr      z,bossChaseTarget
        ld      ix,p2x
bossChaseTarget:
        ; first move on x-axis (if required/possible)
        ld      a,(frame)
        bit     0,a
        jr      z,bossChaseY                    ; only update x-axis every 2nd frame
        ld      hl,(bossX)
        push    hl
        ld      de,7
        add     hl,de                           ; HL = bossX+7
#ifdef TI84PLUSCE
        ld      de,0
#endif
        ld      e,(ix+PX)
        ld      d,(ix+PX+1)
        inc     de
        inc     de
        inc     de                              ; DE = playerX+3
        or      a                               ; clear carry
        sbc     hl,de                           ; L = difference (H is irrelevant every if negative)
        ld      a,l
        ld      b,0                             ; B = direction is left (if movement required)
        jr      nc,bossChaseX
        neg
        inc     b                               ; B = direction is right
bossChaseX:
        cp      13
        pop     hl                              ; HL = bossX
        jr      c,bossChaseY                    ; if X difference less than 12, no need to move on x-axis
        ld      a,b
        or      a
        jr      nz,bossChaseRight
bossChaseLeft:
        dec     hl                              ; HL = bossX-1
        call    bossCheckHoriz
        jr      nc,bossChaseY                   ; if hit a wall, no moving left
        ld      (bossX),hl
        call    bossDidMove
        jr      bossChaseY
bossChaseRight:
        ld      de,16
        add     hl,de                           ; HL = bossX+16
        call    bossCheckHoriz
        jr      nc,bossChaseY                   ; if hit a wall, no moving right
        ld      hl,(bossX)
        inc     hl
        ld      (bossX),hl                      ; updated bossX
        call    bossDidMove
bossChaseY:
        ld      hl,(bossY)
        push    hl
        ld      de,24
        add     hl,de                           ; HL = bossY+16
        call    bossCheckVert
        pop     hl                              ; HL = bossY
        jr      nc,bossChaseDone
        inc     hl
        ld      (bossY),hl
        call    bossDidMove
        jr      moveBossDone
bossChaseDone:
        ld      a,1
        ld      (bossState),a                   ; boss to start lurking next frame
        ret

bossLurk:
        ld      a,(p1dd)
        ld      hl,(p2dd)
        or      l
        jr      nz,bossLurkDone                 ; if either player is dead, stop lurking
        ld      ix,p1x
        call    checkPlayerBehindBoss           ; is player 1 behind the boss?
        jr      nc,bossLurkDone                 ; if so, stop lurking
        ld      ix,p2x
        call    checkPlayerBehindBoss
        jr      nc,bossLurkDone                 ; ditto for player 2
        ld      b,100
        ld      a,(bossShot)
        or      a
        jr      z,bossLurkRandom
        ld      b,10                            ; if boss was shot in the last frame, greater chance of it starting to retreat
bossLurkRandom:
        call    getRandom
        or      a
        ret     nz                              ; only finish lurking if A=0
bossLurkDone:
        ld      a,2
        ld      (bossState),a                   ; boss to start retreating next frame
        ret

checkPlayerBehindBoss:
        ld      a,(ix+PP)
        or      a                               ; is player active?
        scf
        ret     z                               ; if not, ret with carry set (to denote player isn't behind boss)
#ifdef TI84PLUSCE
        ld      de,0
#endif
        ld      e,(ix+PY)
        ld      d,(ix+PY+1)
        ld      hl,(bossY)
        or      a
        sbc     hl,de
        ret                                     ; ret with carry result

moveBossDone:
        ld      a,(bossMoved)
        or      a
        ret     z                               ; only animate if boss moved this frame
        ld      hl,bossSprCnt
        inc     (hl)
        ret

bossDidMove:
        ld      a,1
        ld      (bossMoved),a
        ret
bossCheckHoriz:
        push    hl
        ld      de,(bossY)
        push    de
        ex      de,hl                           ; DE = x coord to check, HL = bossY
        call    getTile
        cp      TILE_SOLID
        pop     hl                              ; HL = bossY
        pop     de                              ; DE = x
        ret     nc                              ; if a wall, can't move left (return the NC)
        ld      bc,15
        add     hl,bc                           ; HL = bossY+15
        push    de
        call    getTile
        cp      TILE_SOLID
        pop     hl                              ; HL = x
        ret                                     ; ret the NC if hit a wall, otherwise will return CA
bossCheckVert:
        push    hl
        ld      de,(bossX)
        push    de
        call    getTile
        cp      TILE_SOLID
        pop     hl                              ; HL = bossX
        pop     de                              ; DE = bossY-1
        ret     nc                              ; if boss hit a wall, it's done retreating
        ld      bc,15
        add     hl,bc
        ex      de,hl                           ; DE = bossX+15, HL = bossY-1
        push    hl
        call    getTile
        cp      TILE_SOLID
        pop     hl                              ; HL = bossY-1
        ret

.end
