;===============================================================;
;                                                               ;
; Alien Breed 5                                                 ;
; Enemy routines                                                ;
;                                                               ;
;===============================================================;

;------------------------------------------------
; checkEnemyTrigger - check a tile to see if it's an enemy trigger
;   input:  HL => tile in level data
;   output: none
;------------------------------------------------
checkEnemyTrigger:
        ld      a,(hl)                          ; A = tile
        cp      TILE_OPENALIENHOLE              ; check if it's an open hole
        jp      z,loadAlienHole                 ; if so, try to load it
        ex      de,hl                           ; DE => tile in level data
        sub     TILE_ENEMYTRIGGER               ; check first enemy trigger
        jr      z,newEnemy1                     ; if so, try to use it
        dec     a                               ; check second enemy trigger
        ret     nz                              ; if not, simply leave
newEnemy2:
        ld      hl,enemy2                       ; HL => enemy type/level
        jr      cetCheckGuard
newEnemy1:
        ld      hl,enemy1
cetCheckGuard:
        ld      a,(hl)
        and     $1F
        cp      7                               ; is the enemy a guard?
        jr      nz,newEnemy                     ; if not, then spawn as normal
        ; guard enemy triggers treated like an open alien hole
        ex      de,hl                           ; HL => tile in level data
        jp      loadAlienHole

;------------------------------------------------
; newEnemy - try to create a new enemy
;   input:  HL => enemy type/level
;           DE => trigger tile in level data
;   output: none
;------------------------------------------------
newEnemy:
        ; check for no enemies cheat
        ld      a,(e2cheatFlags)
        bit     noEnemiesCheat,a
        ret     nz                              ; leave if the cheat is on
        ; check we're not in a boss fight
        ld      a,(state)
        or      a
        ret     nz
        ; check to make sure that enemy isn't spawning on an already existing enemy
        push    hl
        push    de
        ex      de,hl                           ; HL => trigger tile
        call    getExactXYFromHL
        ld      (enemyXTest),de
        ld      (enemyYTest),hl
        ld      a,1
        ld      (enemyCollCheck),a
        call    checkEnemyCollisions
        pop     de
        pop     hl
        ret     nc
        push    ix
        push    de                              ; save ptr to tile in level data
        ; try to find an empty entry in enemyTable
        ld      ix,enemyTable                   ; IX => start of enemyTable
        ld      b,NUM_ENEMIES                   ; B = number of entries to check
        ld      de,ENEMY_SIZE                   ; DE = size of enemyTable entry
findEmptyEnemyEntry:
        ld      a,(ix+0)                        ; A = enemy type
        or      a                               ; 0?
        jr      z,enemyEntryFound               ; if so, empty entry found
        add     ix,de                           ; IX => next entry
        djnz    findEmptyEnemyEntry             ; loop until whole table checked
        pop     de                              ; clear stack
        jp      endNewEnemy                     ; no empty entry found, so skip to end and leave
enemyEntryFound:
        ld      a,(hl)                          ; A = enemy data
        and     %00011111                       ; A = enemy type (0->..)
        inc     a                               ; A = enemy type (1->..)
        ld      (ix+0),a                        ; save it
        ld      b,a                             ; B = enemy type
        ld      (ix+1),%10000000                ; set bit 7 to indicate that a new direction needs to be chosen
        ex      (sp),hl                         ; (SP) => enemy data; HL => tile in level data
        call    getExactXYFromHL                ; DE = x; HL = y
        ld      (ix+2),e
        ld      (ix+3),d                        ; save x
        ld      (ix+4),l
        ld      (ix+5),h                        ; save y
        pop     hl                              ; HL => enemy data
        ld      a,(hl)
        rlca
        rlca
        rlca
        and     %00000111                       ; A = enemy level
        ld      c,a                             ; C = enemy level
        ld      a,b                             ; A = enemy type (B is still preserved from earlier)
        dec     a                               ; A = enemy type (0->..)
        add     a,a
        add     a,a
        add     a,a                             ; *8
        add     a,c                             ; A = (enemy type*8) + enemy level, aka offset for enemyLevelTable
#ifdef TI83PLUS
        ld      d,0
#endif
#ifdef TI84PLUSCE
        ld      de,0
#endif
        ld      e,a                             ; DE = offset for enemyLevelTable
        ld      hl,enemyLevelTable
        add     hl,de
        ld      a,(difficulty)
        or      a                               ; check if NORMAL or TOUGH difficulty
        ld      a,(hl)                          ; A = ini health for enemy
        jr      z,newEnemyAfterTough            ; if NORMAL, don't double ini health
        add     a,a
newEnemyAfterTough:
        ld      (ix+7),a                        ; save it
        ld      l,b                             ; L = enemy type (B is still preserved from earlier)
        dec     l
#ifdef TI83PLUS
        ld      h,0                             ; HL = enemy type (0->..)
        add     hl,hl
        add     hl,hl
        add     hl,hl                           ; *8
#endif
#ifdef TI84PLUSCE
        ld      h,8
        mlt     hl
#endif
        ld      b,8
        call    getRandom                       ; A = 0-7 (random)
#ifdef TI83PLUS
        ld      d,0
#endif
#ifdef TI84PLUSCE
        ld      de,0
#endif
        ld      e,a
        add     hl,de                           ; HL = (enemy type * 8) + random 0-7
        ld      de,enemySpeedTable
        add     hl,de
        ld      a,(hl)                          ; A = enemy speed (picked randomly from enemySpeedTable)
        ld      hl,e1cheatFlags
        bit     fastEnemyCheat,(hl)
        jr      z,checkEnemySlowCheat
        ld      a,1                             ; if enemy fast cheat is on, enemy always moves at full speed
        jr      afterEnemySpeedCheats
checkEnemySlowCheat:
        ld      hl,e2CheatFlags
        bit     slowEnemyCheat,(hl)
        jr      z,afterEnemySpeedCheats
        ld      a,3                             ; if enemy slow cheat is on, enemy always moves slow
afterEnemySpeedCheats:
        ld      b,a                             ; B = enemy speed
        add     a,a
        add     a,a                             ; A = enemy speed shifted left twice
        or      b                               ; A = enemy speed stored twice, in bits 0/1 & bits 2/3
        ld      (ix+8),a                        ; save it
        call    enemyChooseTarget               ; A = target
        ld      (ix+9),a                        ; set initial target
        ; show shooting tip
        ; if tip already shown, don't try to show again
        ; normally this check is done in bcall(ab5_showTip), but we're doing it here to avoid the bcall if possible
        ld      hl,(pTips)
        bit     TIP_SHOOT,(hl)
        ld      a,TIP_SHOOT
        ld      d,0
        bcallz(ab5_showTip)
endNewEnemy:
        pop     ix
        ret

;------------------------------------------------
; loadAlienHole - try load set up data for an alien hole
;   input:  HL => tile in level data where alien hole is located
;   output: none
;------------------------------------------------
loadAlienHole:
        push    ix                              ; IX must be preserved in case one of the horizontal scrolling routines led here
        ld      b,NUM_AHOLES                    ; B = number of entries in aholeTable
        ld      ix,aholeTable
        ld      de,AHOLE_SIZE
findEmptyAholeEntry:
        ld      a,(ix+0)
        or      a                               ; is this entry empty?
        jr      z,aholeEntryFound
        add     ix,de                           ; IX => next entry
        djnz    findEmptyAholeEntry
        pop     ix
        ret                                     ; if no empty entry, leave
aholeEntryFound:
        ld      a,(hl)                          ; A = tile
        cp      TILE_OPENALIENHOLE              ; is it an already open hole, or an enemy trigger in the case of the guard enemy?
        ld      a,INI_OPENALIENHOLE             ; A = initial value for counter in the event of an already open hole
        jr      nc,finishLoadAlienHole          ; if hole is open, go ahead
        ld      a,INI_UNOPENALIENHOLE           ; A = initial value for counter in the even of a hole that isn't open yet
finishLoadAlienHole:
        ld      (ix+0),a                        ; initialise counter
#ifdef TI83PLUS
        ld      (ix+1),l
        ld      (ix+2),h                        ; save ptr to tile in level data
#endif
#ifdef TI84PLUSCE
        ld      (ix+1),hl
#endif
        call    getExactXYFromHL                ; DE = x; HL = y
        ld      (ix+4),e
        ld      (ix+5),d                        ; save x
        ld      (ix+6),l
        ld      (ix+7),h                        ; save y
        pop     ix
        ret

;------------------------------------------------
; updateAlienHoles - update all active alien holes
;   input:  none
;   output: none
;------------------------------------------------
updateAlienHoles:
        ld      hl,frame
        bit     0,(hl)                          ; check if even frame
        ret     nz                              ; if not, don't update this frame
        ld      b,NUM_AHOLES                    ; B = number of entries in aholeTable
        ld      ix,aholeTable                   ; IX => start of alien hole data
updateAholeLoop:
        push    bc                              ; save counter
        ld      a,(ix+0)
        or      a                               ; is this alien hole active
        jr      z,endUpdateAholeLoop            ; if not, skip
        push    ix
        ld      ix,collideObject1
        call    loadScreenCollideObjectIX
        pop     ix
        inc     ix
        inc     ix                              ; IX+2 => coord data for alien hole (bit of a hack to use the next routine)
        call    loadEnemyCollideObject2
        dec     ix
        dec     ix                              ; return IX to start of entry
        call    checkCollision                  ; check if alien hole is still on screen
        jr      nc,clearAlienHole               ; if not, deactivate the alien hole for now
        bit     7,(ix+0)                        ; see if counter is a negative value
        jr      nz,updateUnopenAlienHole        ; if so, hole isn't open yet, so update that part
        dec     (ix+0)                          ; decrement counter
        jr      nz,endUpdateAholeLoop           ; if it hasn't reached zero, nothing else to do this frame
        ; otherwise, if it's reached zero, spawn an alien and reset counter
        ld      (ix+0),INI_OPENALIENHOLE        ; reset counter
        ld      hl,enemy1                       ; HL => enemy type/level (alien hole spawns first enemy type)
#ifdef TI83PLUS
        ld      e,(ix+1)
        ld      d,(ix+2)                        ; DE => tile in level data
#endif
#ifdef TI84PLUSCE
        ld      de,(ix+1)
#endif
        call    newEnemy                        ; load a new enemy
        jr      endUpdateAholeLoop
clearAlienHole:
        ld      (ix+0),0                        ; turn off alien hole
        jr      endUpdateAholeLoop
updateUnopenAlienHole:
        inc     (ix+0)                          ; increment counter
        jr      nz,endUpdateAholeLoop           ; if it hasn't reached zero, nothing else to do this frame
        ; otherwise, if it's reached zero, break the hole open and reset the counter for an open hole
        ld      (ix+0),INI_OPENALIENHOLE        ; reset counter
#ifdef TI83PLUS
        ld      l,(ix+1)
        ld      h,(ix+2)                        ; HL => alien hole in level data
#endif
#ifdef TI84PLUSCE
        ld      hl,(ix+1)
#endif
        ld      a,TILE_OPENALIENHOLE            ; A = tile to set
        push    ix                              ; save data ptr
        call    putTile                         ; put the tile & display it
        pop     ix                              ; restore data ptr
endUpdateAholeLoop:
        ld      de,AHOLE_SIZE
        add     ix,de                           ; IX => next entry
        pop     bc                              ; restore counter
        djnz    updateAholeLoop
        ret

;------------------------------------------------
; drawEnemies - draw enemies from enemyTable
;   input:  none
;   output: none
;------------------------------------------------
drawEnemies:
        ld      ix,enemyTable                   ; IX => first entry
        ld      b,NUM_ENEMIES                   ; B = num entries to check/draw
drawEnemiesLoop:
        push    bc                              ; save loop counter
        ld      a,(ix+0)                        ; A = enemy type
        or      a                               ; 0?
        jr      z,endDrawEnemiesLoop            ; if so, no enemy in this entry
        dec     a
#ifdef TI83PLUS
        ld      l,a
        ld      h,0
        ld      e,l
        ld      d,h                             ; DE & HL both = enemy type - 1
        add     hl,hl                           ; *2
#endif
#ifdef TI84PLUSCE
        LDHLA()
        push hl \ pop de
        ld      h,3
        mlt     hl
#endif
        ld      bc,enemySpriteTable
        add     hl,bc
        LDHLIND()                               ; HL => first sprite for this enemy type
        ex      de,hl                           ; DE => first sprite; HL = enemy type - 1
        ld      bc,enemyCounterMask
        add     hl,bc
        ld      a,(hl)                          ; A = mask to apply to enemy counter
        and     (ix+6)                          ; A = enemy counter masked, which becomes offset factor for animated sprite
#ifdef TI83PLUS
        ld      l,a
        ld      h,0
        add     hl,hl
        add     hl,hl
        add     hl,hl
        add     hl,hl
        add     hl,hl                           ; *32 = offset to animated sprite
#endif
#ifdef TI84PLUSCE
        ld      l,a
        ld      h,128
        mlt     hl
        add     hl,hl
        add     hl,hl
        add     hl,hl                           ; x1024
#endif
        add     hl,de                           ; HL => sprite set to show, now to calc direction
        ld      a,(ix+1)                        ; A = enemy direction
        and     %01111111                       ; ignore bit 7 (it's a flag)
#ifdef TI83PLUS
        add     a,a
        add     a,a
        add     a,a                             ; *8
        ld      e,a
        ld      d,0
#endif
#ifdef TI84PLUSCE
        ex      de,hl
        ld      l,a
        ld      h,128
        mlt     hl
        add     hl,hl
#endif
        add     hl,de                           ; HL => sprite to show
        push    hl                              ; save sprite ptr
        ld      de,(screenX)
#ifdef TI84PLUSCE
        or a \ sbc hl,hl
#endif
        ld      l,(ix+2)
        ld      h,(ix+3)                        ; HL = enemy x
        or      a                               ; clear carry
        sbc     hl,de                           ; L = x coord to display at (H will be 0)
        ld      b,l                             ; B = x
        ld      de,(screenY)
#ifdef TI84PLUSCE
        or a \ sbc hl,hl
#endif
        ld      l,(ix+4)
        ld      h,(ix+5)                        ; HL = enemy y
        or      a
        sbc     hl,de                           ; L = y coord to display at (H will be 0)
        ld      c,l                             ; C = y
        pop     hl                              ; HL => sprite
        push    ix                              ; save enemyTable ptr
#ifdef TI83PLUS
        call    putClippedSprite                ; draw sprite
#endif
#ifdef TI84PLUSCE
        ld      e,b
        ld      d,c
        ld      a,(ix+0)
        cp      ENEMY_CLOAKED+1
        jr      nz,drawEnemyNormal
        call    drawClippedTransSpriteCloaked
        jr      drawEnemiesAfterDraw
drawEnemyNormal:
        call    drawClippedTransSprite
#endif
drawEnemiesAfterDraw:
        pop     ix                              ; restore enemyTable ptr
endDrawEnemiesLoop:
        pop     bc                              ; restore loop counter
        ld      de,ENEMY_SIZE
        add     ix,de                           ; IX => next entry
#ifdef TI83PLUS
        djnz    drawEnemiesLoop
#endif
#ifdef TI84PLUSCE
        dec     b
        jp      nz,drawEnemiesLoop
#endif
        ret

;------------------------------------------------
; moveEnemies - move enemies
;   input:  none
;   output: none
;------------------------------------------------
moveEnemies:
        ld      a,2
        ld      (enemyCollCheck),a              ; set this value for checkEnemyCollisions
        ld      ix,collideObject1
        call    loadScreenCollideObjectIX       ; will be used to check if enemies are still on screen
        ld      ix,enemyTable                   ; IX => start of enemy data
        ld      b,NUM_ENEMIES                   ; B = number enemies to check/move
moveEnemiesLoop:
        push    bc                              ; save counter
        ld      a,(ix+0)                        ; A = enemy type
        or      a                               ; 0?
        jp      z,endMoveEnemiesLoop            ; if so, no enemy in this entry
        ld      hl,e3CheatFlags
        bit     boomersCheat,(hl)
        jr      z,moveEnemiesNoBoomers
        ld      b,200
        call    getRandom
        or      a
        jr      nz,moveEnemiesNoBoomers
        call    enemyBoomer
        jp      endMoveEnemiesLoop
moveEnemiesNoBoomers:
        ; check what tile the enemy is currently standing on
        ; this will affect wall collisions
#ifdef TI84PLUSCE
        or a \ sbc hl,hl
        push hl \ pop de
#endif
        ld      e,(ix+2)
        ld      d,(ix+3)
        inc     de
        inc     de
        inc     de                              ; DE = enemy x centre
        ld      l,(ix+4)
        ld      h,(ix+5)
        inc     hl
        inc     hl
        inc     hl                              ; HL = enemy y centre
        call    getTile                         ; A = tile at centre of enemy
        call    checkE2Trigger                  ; if the tile is an enemy 2 trigger, check the tile to the right instead
        ld      (enemyCt),a                     ; save the tile that the enemy is standing on
        call    loadEnemyCollideObject2         ; load enemy position into collideObject2
        call    checkCollision                  ; check if enemy is "colliding" with the screen, ie. check if the enemy is on screen or just outside screen
        jr      c,enemyOnScreen                 ; if so, continue with movement
        ld      (ix+0),0                        ; otherwise, disable this enemy entry
        jp      endMoveEnemiesLoop              ; and go to next entry
enemyOnScreen:
        ld      a,(ix+8)                        ; A = speed counter & speed counter max, in bits 0/1 & bits 2/3 respectively
        or      a
        jr      z,enemyNewDirection             ; if speed = 0, this enemy doesn't move (turrets), but it does adjust it's direction
        ld      c,a                             ; save it to C
        and     %00000011
        ld      b,a                             ; B = speed counter
        ld      a,c
        and     %00001100                       ; A = speed counter max in bits 2/3
        dec     b                               ; dec counter
        jp      nz,noMoveEnemy                  ; if NZ, don't move this frame
        ld      b,a                             ; B = speed counter max in bits 2/3
        srl     b
        srl     b                               ; B = speed counter max in bits 0/1
        or      b                               ; A = updated value of speed counter & max, in bits 0/1 & 2/3 respectively
        ld      (ix+8),a                        ; save it
        ; now work out enemy movement
        ld      a,(ix+1)                        ; A = enemy direction
        bit     7,a                             ; check if it's set or null
        jr      z,moveEnemy                     ; if it's set, try to move enemy in set direction
        ; otherwise, work out a new direction for enemy
enemyNewDirection:
        ld      a,(ix+9)                        ; A = target
        ld      hl,(p1x)
        ld      de,(p1y)
        or      a
        jr      z,chooseEnemyDirection
        ld      hl,(p2x)
        ld      de,(p2y)
chooseEnemyDirection:
#ifdef TI84PLUSCE
        ld      bc,0
#endif
        ld      c,(ix+2)
        ld      b,(ix+3)                        ; BC = enemy x
        or      a
        sbc     hl,bc                           ; get difference between player & enemy x coords
        ld      (tempX),hl                      ; save x difference
        ex      de,hl                           ; DE = x diff, HL = player y
#ifdef TI84PLUSCE
        ld      bc,0
#endif
        ld      c,(ix+4)
        ld      b,(ix+5)                        ; BC = enemy y
        or      a
        sbc     hl,bc                           ; get y coord difference
        ld      (tempY),hl                      ; save y difference
        call    absHL                           ; HL = abs(HL)
        ex      de,hl                           ; DE = abs(y diff); HL = x diff
        call    absHL                           ; HL = abs(x diff)
        or      a
        sbc     hl,de                           ; check which difference is greater
        jr      c,enemyYGreater                 ; if y difference greater, try to move on y axis
        ld      hl,(tempX)                      ; otherwise, try to move on x axis
        bit     7,h                             ; check if should move left or right
        jr      z,setEnemyDirRight
        ld      a,2                             ; A = left
        jr      moveEnemyCheat
setEnemyDirRight:
        ld      a,3                             ; A = right
        jr      moveEnemyCheat
enemyYGreater:
        ld      hl,(tempY)
        bit     7,h                             ; check if should move up or down
        jr      z,setEnemyDirDown
        xor     a                               ; A = up
        jr      moveEnemyCheat
setEnemyDirDown:
        ld      a,1                             ; A = down

moveEnemyCheat:
        ld      hl,e3CheatFlags
        bit     muntersCheat,(hl)
        jr      z,moveEnemy
        xor     1
moveEnemy:
        ld      (ix+1),a                        ; save enemy direction
        ld      a,(ix+8)
        or      a
        jp      z,endMoveEnemiesLoop            ; if speed = 0, don't actually move (turret)
        ld      a,(ix+1)
        ld      hl,enemyMoveDone                ; HL => where to ret to
        push    hl                              ; push address to stack
        or      a
        jr      z,moveEnemyUp
        dec     a
        jp      z,moveEnemyDown
        dec     a
        jp      z,moveEnemyLeft

moveEnemyRight:
#ifdef TI84PLUSCE
        or a \ sbc hl,hl
#endif
        ld      l,(ix+2)
        ld      h,(ix+3)
        inc     hl                              ; HL = prospective new x
        ld      (enemyXTest),hl                 ; save it for collision test
        ld      de,7
        add     hl,de
        ex      de,hl                           ; DE = x coord of tile to check
moveEnemyX:
        ld      hl,miscFlags
        set     enemyPlane,(hl)
#ifdef TI84PLUSCE
        or a \ sbc hl,hl
#endif
        ld      l,(ix+4)
        ld      h,(ix+5)                        ; HL = enemy y
        ld      (enemyYTest),hl                 ; save it for collision test
        call    enemyCheckTile                  ; check if enemy can walk here
        ret     nc                              ; if not, leave
        ld      bc,7
        add     hl,bc                           ; HL = enemy y + 7 (bottom corner)
        call    enemyCheckTile
        ret     nc
        call    checkEnemyCollisions            ; make sure enemy isn't colliding with other enemies
        ret     nc
        ld      a,(enemyXTest)                  ; A = lsb of new x coord

enemyMoveSuccess:
        inc     (ix+6)                          ; inc enemy counter
        ld      hl,(enemyXTest)                 ; HL = new enemy x
        ld      (ix+2),l
        ld      (ix+3),h                        ; save it
        ld      hl,(enemyYTest)
        ld      (ix+4),l
        ld      (ix+5),h
        and     %00000111                       ; check if enemy has moved on to an even tile (A was either lsb of new x or y coord depending on the plane that has changed)
        scf                                     ; tell the moveEnemyDone routine that the move was a success
        ret     nz                              ; if not even tile, nothing more to do
        ld      a,(ix+1)
        or      %10000000                       ; set bit 7 to indicate that next frame the enemy should choose a new direction to move
        ld      (ix+1),a
        scf                                     ; again with the scf (see a few lines above)
        ret

moveEnemyUp:
#ifdef TI84PLUSCE
        or a \ sbc hl,hl
#endif
        ld      l,(ix+4)
        ld      h,(ix+5)
        dec     hl
        ld      (enemyYTest),hl
moveEnemyY:
        push    hl
        ld      hl,miscFlags
        res     enemyPlane,(hl)
        pop     hl
#ifdef TI84PLUSCE
        ld      de,0
#endif
        ld      e,(ix+2)
        ld      d,(ix+3)
        ld      (enemyXTest),de
        call    enemyCheckTile
        ret     nc
        ex      de,hl                           ; DE = y, HL = x
        ld      bc,7
        add     hl,bc
        ex      de,hl                           ; DE = x+7, HL = y
        call    enemyCheckTile
        ret     nc
        call    checkEnemyCollisions
        ret     nc
        ld      a,(enemyYTest)                  ; A = lsb of new y coord
        jr      enemyMoveSuccess

moveEnemyDown:
#ifdef TI84PLUSCE
        or a \ sbc hl,hl
#endif
        ld      l,(ix+4)
        ld      h,(ix+5)
        inc     hl
        ld      (enemyYTest),hl
        ld      de,7
        add     hl,de
        jr      moveEnemyY

moveEnemyLeft:
#ifdef TI84PLUSCE
        ld      de,0
#endif
        ld      e,(ix+2)
        ld      d,(ix+3)
        dec     de
        ld      (enemyXTest),de
        jp      moveEnemyX

enemyMoveDone:
        jp      c,endMoveEnemiesLoop            ; if move was successful, nothing else to do
        ld      hl,enemy2ndMoveDone             ; HL => where to ret to after 2nd move attempt
        push    hl                              ; save ret address to stack
        ld      a,(p1p)
        ld      hl,p2p
        add     a,(hl)
        cp      2                               ; 2 active players?
        call    z,moveEnemy2P                   ; if so, see if both players are near each other
        ld      hl,miscFlags
        bit     enemyPlane,(hl)
        jr      z,enemyMoveTryX                 ; if z, check the x plane for movement
        ld      a,(ix+9)
        ld      hl,(p1y)
        or      a
        jr      z,enemyMoveTryYCalc
        ld      hl,(p2y)
enemyMoveTryYCalc:
#ifdef TI84PLUSCE
        ld      de,0
#endif
        ld      e,(ix+4)
        ld      d,(ix+5)
        or      a
        sbc     hl,de                           ; HL = y coord difference
        bit     7,h                             ; check if pos or neg
        ld      (ix+1),0                        ; set direction to up in case enemy is moving up
        jp      nz,moveEnemyUp
        ld      (ix+1),1
        jr      moveEnemyDown
enemyMoveTryX:
        ld      a,(ix+9)
        ld      hl,(p1x)
        or      a
        jr      z,enemyMoveTryXCalc
        ld      hl,(p2x)
enemyMoveTryXCalc:
#ifdef TI84PLUSCE
        ld      de,0
#endif
        ld      e,(ix+2)
        ld      d,(ix+3)
        or      a
        sbc     hl,de
        bit     7,h
        ld      (ix+1),2
        jp      nz,moveEnemyLeft
        ld      (ix+1),3
        jp      moveEnemyRight

enemy2ndMoveDone:
        jr      c,endMoveEnemiesLoop
        ld      b,24
        call    getRandom                       ; get a random direction (or anything higher than 3 means wait next frame)
        cp      4
        jr      c,enemySetRandomDir
        ld      a,(ix+1)
        or      %10000000
enemySetRandomDir:
        ld      (ix+1),a                        ; save it to be tried next time
        call    enemyChooseTarget
        ld      (ix+9),a

endMoveEnemiesLoop:
        pop     bc                              ; restore counter
        ld      de,ENEMY_SIZE
        add     ix,de                           ; IX => next enemy entry
        dec     b
        jp      nz,moveEnemiesLoop              ; same as djnz, but to accommodate to longer jump distance
        ret

noMoveEnemy:
        or      b                               ; A = updated value of speed counter & max, in bits 0/1 & 2/3 respectively
        ld      (ix+8),a                        ; save it
        jr      endMoveEnemiesLoop              ; go to end of loop

moveEnemy2P:
        push    ix
        ld      ix,p1x
        ld      hl,collideObject1
        call    loadPlayerCollideObjectHL
        ld      ix,p2x
        ld      hl,collideObject2
        call    loadPlayerCollideObjectHL
        call    checkCollision                  ; check if players are intersecting
        push    af                              ; save result
        ld      ix,collideObject1
        call    loadScreenCollideObjectIX
        pop     af                              ; get result again
        pop     ix
        ret     c                               ; if players are intersecting, ret back and proceed as if a 1P game
        ; otherwise players aren't intersecting, so head to enemy2ndMoveDone
        ; note that carry is already clear
        pop     hl                              ; clear last ret from stack
        ret

;------------------------------------------------
; enemyCheckTile - check a tile to see if enemy can walk on it
;   input:  DE = x
;           HL = y
;   output: CA = can walk on this tile
;           DE = x
;           HL = y
;------------------------------------------------
enemyCheckTile:
        push    de                              ; save x
        push    hl                              ; save y
        call    getTile                         ; A = tile
        call    checkE2Trigger
        ld      b,a                             ; B = tile
        ld      a,(enemyCt)                     ; A = tile that enemy is currently standing on
        or      a
        jr      z,ectServiceDuct                ; handle for service duct
        cp      TILE_FLOOR
        jr      c,ectRamp                       ; handle for ramps
        ; otherwise handle for normal floor
        ld      a,b
        or      a
        jr      z,enemyCheckTileDone            ; carry will be cleared, can't walk directly on to service duct
        cp      TILE_SOLID
        jr      enemyCheckTileDone
ectServiceDuct:
        ld      a,b
        cp      TILE_FLOOR
        jr      enemyCheckTileDone
ectRamp:
        ld      a,b
        cp      TILE_SOLID
enemyCheckTileDone:
        pop     hl                              ; HL = y
        pop     de                              ; DE = x
        ret

;------------------------------------------------
; enemyDead - kill an enemy and initiate an explosion animation
;   input:  IX => entry in enemyTable
;           (pPlayer) => player data
;   output: IX => entry in enemyTable
;------------------------------------------------
enemyDead:
        push    ix
        ld      a,(ix)
        cp      7                               ; was enemy a turret?
        jr      nz,enemyDeadContinue            ; if not, skip this next bit
        ; replace enemy trigger tile with a crater
#ifdef TI84PLUSCE
        ld de,0 \ or a \ sbc hl,hl
#endif
        ld      e,(ix+2)
        ld      d,(ix+3)
        ld      l,(ix+4)
        ld      h,(ix+5)
        call    getTile
        ld      a,DIRT_CRATER
        call    putTile
enemyDeadContinue:
        ld      hl,SCORE_ENEMY
        ld      ix,(pPlayer)
        call    addScoreDiff                    ; add points for killing enemy
        ; inc enemy kills achievements
        ld      iy,flags
        ld      bc,ACH_ENEMY1_BIT*256+ACH_ENEMY1_ID
        ld      de,ACH_ENEMY1_OFFSET
        ld      hl,1
        bcall(ab5_incAchievement)
        ld      bc,ACH_ENEMY2_BIT*256+ACH_ENEMY2_ID
        ld      de,ACH_ENEMY2_OFFSET
        ld      hl,1
        bcall(ab5_incAchievement)
        ld      bc,ACH_ENEMY3_BIT*256+ACH_ENEMY3_ID
        ld      de,ACH_ENEMY3_OFFSET
        ld      hl,1
        bcall(ab5_incAchievement)
        ld      bc,ACH_ENEMY4_BIT*256+ACH_ENEMY4_ID
        ld      de,ACH_ENEMY4_OFFSET
        ld      hl,1
        bcall(ab5_incAchievement)
        ld      bc,ACH_ENEMY5_BIT*256+ACH_ENEMY5_ID
        ld      de,ACH_ENEMY5_OFFSET
        ld      hl,1
        bcall(ab5_incAchievement)
        pop     ix
enemyBoomer:
        ld      (ix+0),0                        ; clear enemy type
        ld      e,(ix+2)
        ld      d,(ix+3)                        ; DE = x
        ld      l,(ix+4)
        ld      h,(ix+5)                        ; HL = y
        ld      bc,0*256+ANIM_EXPLOSION         ; B = animation type; C = counter ini
        jp      newAnim                         ; set up new animation

;------------------------------------------------
; enemyChooseTarget - choose which player the enemy will follow
;   input:  IX => enemyTable entry
;   output: IX => enemyTable entry
;           A = 0,1 (player 1 or player 2 respectively)
;------------------------------------------------
enemyChooseTarget:
        ld      a,(p1p)
        or      a
        jr      z,ectP2                         ; if player 1 not active, must follow player 2
        ld      a,(p2p)
        or      a
        ret     z                               ; if player 2 not active, must follow player 1 (A already equals 0 to denote this)
        ld      b,2
        call    getRandom                       ; otherwise if both players active, choose randomly
        ret
ectP2:
        inc     a                               ; A = 1
        ret

;------------------------------------------------
; despawnEnemies - despawn all enemies (for a boss fight)
;   input:  none
;   output: none
;------------------------------------------------
despawnEnemies:
        ld      b,NUM_ENEMIES
        ld      ix,enemyTable
despawnEnemiesLoop:
        push    bc
        ld      a,(ix)
        or      a
        jr      z,endDespawnEnemiesLoop
        ld      (ix+0),0                        ; clear enemy type
        ld      e,(ix+2)
        ld      d,(ix+3)                        ; DE = x
        ld      l,(ix+4)
        ld      h,(ix+5)                        ; HL = y
        ld      bc,0*256+ANIM_EXPLOSION         ; B = animation type; C = counter ini
        call    newAnim                         ; set up new animation
endDespawnEnemiesLoop:
        ld      de,ENEMY_SIZE
        add     ix,de
        pop     bc
        djnz    despawnEnemiesLoop
        ret

.end
