;===============================================================;
;                                                               ;
; Alien Breed 5                                                 ;
; Intex routines                                                ;
;                                                               ;
;===============================================================;

;------------------------------------------------
; loadIntexConsole - start the intex console routines
;   input:  none
;   output: none
;------------------------------------------------
loadIntexConsole:
        ; wait for [ALPHA] to be released
#ifdef TI83PLUS
        ld      a,KG_1
        bcall(ab5_directIn)
        bit     DI_ALPHA,a
        jr      z,loadIntexConsole
        bcall(ab5_saveVideo)                    ; save video buffer
        call    textScreenBg
        ld      hl,strIntexLoad                 ; HL => loading text data
        call    displayPageNoClear
#endif
#ifdef TI84PLUSCE
        call    keyScan
        ld      a,(kbdG2)
        bit     kbitAlpha,a
        jr      nz,loadIntexConsole
        call    fadeOutClear
        ld      hl,strIntexLoad
        call    displayPageNoClear
#endif
        ld      b,40                            ; B = wait count
        call    delay

;------------------------------------------------
; intexMainMenu - prepare for the intex main menu
;   input:  none
;   output: HL => main menu data
;           DE => jump table
;------------------------------------------------
intexMainMenu:
        ld      hl,intexMainMenuData
        ld      de,intexMainJumpTable
        xor     a
        jr      doIntexMenu

;------------------------------------------------
; intexWeaponMenu - prepare for the intex weapon menu
;   input:  none
;   output: HL => weapon menu data
;           DE => jump table
;------------------------------------------------
intexWeaponMenu:
        ld      hl,intexWeaponMenuData
        ld      de,intexWeaponJumpTable
        ld      a,1
        jr      doIntexMenu

;------------------------------------------------
; intexToolMenu - prepare for the intex tool menu
;   input:  none
;   output: HL => tool menu data
;           DE => jump table
;------------------------------------------------
intexToolMenu:
        ld      hl,intexToolMenuData
        ld      de,intexToolJumpTable
        ld      a,1
        ; fall through to menu routine

;------------------------------------------------
; doIntexMenu - execute an intex menu
;   input:  HL => menu data
;           DE => jump table
;           A = 0 if credits not to be shown, otherwise will be shown
;   output: none
;------------------------------------------------
doIntexMenu:
        ld      (intexShowCredits),a
        ld      (intexJumpTable),de             ; save jump table location
        ld      a,(hl)                          ; A = number of options for this menu
        ld      (intexOptions),a                ; save it
        inc     hl                              ; HL => text data
        push    hl
#ifdef TI83PLUS
        call    textScreenBg
#endif
#ifdef TI84PLUSCE
        call    vramClear
#endif
        pop     hl
        call    displayPageNoClear              ; display all text data
#ifdef TI83PLUS
        ld      (intexCursorTable),hl           ; save ptr to cursor position table
#endif
        xor     a
        ld      (intexMenuItem),a               ; reset to top menu item
        ld      a,(intexShowCredits)
        or      a
        jr      z,intexMenuStart                ; if Z, skip showing credit limit
        ld      hl,strCreditLimit
        call    displayPageNoClear              ; type out "CREDIT LIMIT:" without clearing gbuf
#ifdef TI83PLUS
        ld      hl,(credits)
        ld      de,53*256+50
        call    setShowHL                       ; show credits
#endif
#ifdef TI84PLUSCE
        call    copyVram
        ld      hl,(credits)
        ld      de,184
        ld      c,180
        call    setShowHL
        call    vramFlip
#endif

intexMenuStart:
#ifdef TI84PLUSCE
        ld      a,$FF
        call    drawIntexCursor
#endif
intexMenuLoop:
        ld      a,(frame)
        inc     a
        ld      (frame),a
#ifdef TI84PLUSCE
        call    updateCursorFlash
#endif
        ld      b,8
        call    delay
#ifdef TI83PLUS
        call    drawIntexCursor                 ; draw cursor
        bcall(_grbufcpy)                        ; update gbuf to lcd
        call    drawIntexCursor                 ; draw cursor
#endif
        bcall(_getcsc)
        ld      hl,intexMenuItem                ; HL => selected menu item
        cp      GK_UP                           ; [UP]?
        jr      z,intexUp                       ; if so, move up
        cp      GK_DOWN                         ; [DOWN]?
        jr      z,intexDown                     ; if so, move down
        cp      GK_2ND                          ; [2nd]?
        jr      z,intexSelect                   ; if so, process selected option
        cp      GK_ENTER                        ; [ENTER]?
        jr      nz,intexMenuLoop
intexSelect:
        ld      hl,(intexMenuItem)              ; HL = menu item
#ifdef TI83PLUS
        ld      h,0
        add     hl,hl                           ; *2
#endif
#ifdef TI84PLUSCE
        ld      h,3
        mlt     hl
#endif
        ld      de,(intexJumpTable)             ; DE => start of jump table (set from earlier)
        add     hl,de                           ; HL => jump table entry
        LDHLIND()                               ; HL = routine
        jp      (hl)                            ; jump to appropriate routine

intexUp:
        ld      a,(hl)                          ; A = selected menu item
        or      a                               ; already at the top?
        jr      z,intexMenuStart                ; if so, can't go up, just return to loop
#ifdef TI84PLUSCE
        ld      a,COLOUR_BLACK
        call    drawIntexCursor
        ld      hl,intexMenuItem
#endif
        dec     (hl)                            ; dec selected menu item
        jr      intexMenuStart
intexDown:
        ld      a,(intexOptions)                ; A = number of menu options (set from earlier)
        dec     a                               ; A = last menu item
        cp      (hl)                            ; is this what player is currently on?
        jr      z,intexMenuStart                ; if so, can't go down, just return to loop
#ifdef TI84PLUSCE
        ld      a,COLOUR_BLACK
        call    drawIntexCursor
        ld      hl,intexMenuItem
#endif
        inc     (hl)                            ; inc selected menu item
        jr      intexMenuStart

;------------------------------------------------
; intexRadarService - show the level map
;   input:  none
;   output: none
;------------------------------------------------
#ifdef TI83PLUS
intexRadarService:
        ld      hl,intexMainMenu                ; where to ret to after showing map
        push    hl                              ; push it to stack
showMap:
        bcall(_grbufclr)
        call    textScreenBg
        ld      a,(levelHeight)
        push    af                              ; save level height for later
        sub     62
        neg
        srl     a                               ; A = row # of gbuf to start drawing to
        ld      (tempY),a                       ; save this for later
        ld      l,a
        ld      h,12
        bcall(_htimesl)
        ld      de,gbuf
        add     hl,de                           ; HL => row of gbuf to start drawing to
        ld      a,(levelWidth)
        sub     96
        neg
        srl     a                               ; A = pixel # to start at on the row
        ld      (tempX),a                       ; save this for later
        ld      d,a                             ; save it
        and     %00000111                       ; A = {0-7}
        or      %00001000                       ; A = {8-15} (done so that A can't equal 0)
        ld      b,a
        ld      a,%10000000
calcMapMask:
        rrca
        djnz    calcMapMask                     ; rotate A right B times
        pop     bc                              ; B = level height, ie. # rows to draw
        ld      c,a                             ; C = initial value for pixel drawing tool
        ld      a,d                             ; A = pixel # to start at on the row
        srl     a
        srl     a
        srl     a
        ld      d,0
        ld      e,a
        add     hl,de                           ; HL => exact byte to start drawing at on gbuf
        ld      de,(tileDataPtr)                ; DE => start of level tile data
showMapRow:
        push    bc                              ; save row counter
        push    hl                              ; save gbuf ptr
        ld      a,(levelWidth)
        ld      b,a                             ; B = number of pixels to draw in this row
showMapPixel:
        ld      a,(de)                          ; A = tile
        inc     de                              ; DE => next tile
        cp      TILE_SOLID                      ; is it a solid tile?
        jr      c,afterDrawMapPixel             ; if not, don't draw a pixel
        cp      TILE_DOOR                       ; is it a normal door?
        jr      nc,afterDrawMapPixel            ; if so, don't draw a pixel
        ld      a,c                             ; A = pixel drawing tool
        or      (hl)                            ; A = new gbuf byte with new pixel OR'd on
        ld      (hl),a                          ; save it to gbuf
afterDrawMapPixel:
        ld      a,c                             ; A = pixel drawing tool
        rrca                                    ; rotate pixel to the right, set carry if it overflows
        ld      c,a                             ; C = pixel drawing tool with new pixel position
        jr      nc,afterNextMapByte             ; if pixel bit didn't overflow, skip the gbuf inc
        inc     hl                              ; HL => next byte on gbuf
afterNextMapByte:
        djnz    showMapPixel                    ; loop to show another pixel until row is finished
        push    de                              ; save level tile data ptr
        bcall(_grbufcpy)                        ; copy gbuf to lcd after each byte to give a "printing" effect
        ld      b,3
        call    delay
        pop     de                              ; restore level tile data ptr
        pop     hl                              ; restore gbuf ptr (to start of row we just finished)
        ld      bc,12
        add     hl,bc                           ; HL => next gbuf row
        pop     bc                              ; restore row counter
        djnz    showMapRow                      ; loop to show another row until all rows are done
showMapFlashLoop:
        ; use a bit of a hack of other routines to get position to draw marker
        ld      hl,sprMapMarker
        call    spriteToRam                     ; load sprite to ram
        ld      de,(p1x)
        inc     de
        inc     de
        inc     de                              ; DE = x coord of centre of player
        ld      hl,(p1y)
        inc     hl
        inc     hl
        inc     hl                              ; HL = y coord of centre of player
        bcall(ab5_getTile)                      ; HL => tile location in level data of where player is standing
        bcall(ab5_getBlockXYFromHL)             ; A = x (block value); L = y (block value)
        ld      b,a                             ; B = x
        ld      a,l
        ld      hl,tempY
        add     a,(hl)
        dec     a
        ld      c,a                             ; C = y coord to draw cursor at
        ld      a,b
        dec     hl
        dec     hl                              ; HL => (tempX)
        add     a,(hl)
        dec     a                               ; A = x coord
        ld      l,c                             ; L = y coord
        ld      de,sprBuffer                    ; DE => sprite
        bcall(ab5_putSprite)                    ; XOR sprite to gbuf
        bcall(_grbufcpy)                        ; update gbuf to lcd
        ld      b,40                            ; B = delay value
showMapFlashDelay:
        ei
        halt
        push    bc                              ; save delay value
        bcall(_getcsc)                          ; A = any key press
        pop     bc
        cp      GK_2ND                          ; [2nd]?
        ret     z                               ; if so, ret from routine
        cp      GK_ENTER                        ; [ENTER]?
        ret     z
        djnz    showMapFlashDelay               ; continue with delay until B=0
        jr      showMapFlashLoop                ; XOR the marker & repeat
#endif
#ifdef TI84PLUSCE
intexRadarService:
        ld      hl,intexMainMenu                ; where to ret to after showing map
        push    hl                              ; push it to stack
showMap:
        call    vramClear
        ld      a,(levelHeight)
        push    af                              ; save level height for later
        sub     120
        neg                                     ; A = row # of vram to start drawing to
        ld      (tempY),a                       ; save this for later
        ld      l,a
        ld      h,160
        mlt     hl
        add     hl,hl
        ld      de,(pVram)
        add     hl,de                           ; HL => row of vram to start drawing to
        ld      a,(levelWidth)
        sub     160
        neg                                     ; A = pixel # to start at on the row
        ld      (tempX),a                       ; save this for later
        ld      de,0
        ld      e,a
        add     hl,de                           ; HL => where on vram to start drawing to
        pop     af                              ; A = level height
        ld      b,a                             ; B = # rows to draw
        ld      de,(tileDataPtr)                ; DE => start of level tile data
showMapRow:
        push    bc                              ; save row counter
        push    hl                              ; save vram ptr
        ld      a,(levelWidth)
        ld      b,a                             ; B = # of pixels to draw in this row
showMapPixel:
        ld      a,(de)                          ; A = tile
        inc     de                              ; DE => next tile
        cp      TILE_SOLID                      ; is it a solid tile?
        jr      c,afterDrawMapPixel             ; if not, don't draw a pixel
        cp      TILE_DOOR                       ; is it a normal door?
        jr      nc,afterDrawMapPixel            ; if so, don't draw a pixel
        push    hl
        push    de
        ld      a,COLOUR_WHITE
        ld      (hl),a
        inc     hl
        ld      (hl),a
        ld      de,320-1
        add     hl,de
        ld      (hl),a
        inc     hl
        ld      (hl),a
        pop     de
        pop     hl
afterDrawMapPixel:
        inc     hl
        inc     hl
        djnz    showMapPixel                    ; loop to show another pixel until row is finished
        ld      b,2
        call    delay
        pop     hl                              ; restore gbuf ptr (to start of row we just finished)
        ld      bc,640
        add     hl,bc
        pop     bc                              ; restore row counter
        djnz    showMapRow                      ; loop to show another row until all rows are done
        ; draw a square where the player is
        ld      de,(p1x)
        inc     de
        inc     de
        inc     de
        ld      hl,(p1y)
        inc     hl
        inc     hl
        inc     hl
        call    getTile                         ; HL => tile location in level data of where player is standing
        call    getBlockXYFromHL                ; A = x (block value); L = y (block value)
        ld      ix,tempX
        add     a,a
        add     a,(ix)
        sub     2
        ld      de,0
        ld      e,a
        ld      a,l
        add     a,a
        add     a,(ix+3)
        sub     2
        ld      l,a
        ld      h,160
        mlt     hl
        add     hl,hl
        add     hl,de
        ld      de,(pVram)
        add     hl,de
        ld      de,320-6
        ld      c,6
smcOuterLoop:
        ld      b,6
smcInnerLoop:
        ld      (hl),$FF
        inc     hl
        djnz    smcInnerLoop
        add     hl,de
        dec     c
        jr      nz,smcOuterLoop
showMapLoop:
        ld      b,8
        call    delay
        ld      a,(frame)
        inc     a
        ld      (frame),a
        call    updateCursorFlash
        call    _getcsc
        cp      GK_2ND
        ret     z
        cp      GK_ENTER
        ret     z
        jr      showMapLoop
#endif

;------------------------------------------------
; intexMissionObjective - show the mission objectives from the intex console
;   input:  none
;   output: none
;------------------------------------------------
intexMissionObjective:
        call    showObjectives                  ; run the routine to show the mission objectives
#ifdef TI83PLUS
        ld      hl,str2nd
        call    showStrings
        bcall(_grbufcpy)
#endif
        call    wait2ndEnter                    ; wait for [2nd]/[ENTER]
        jp      intexMainMenu                   ; return to main menu

;------------------------------------------------
; intexEntertainment - play the intex entertainment space pong game
;   input:  none
;   output: none
;------------------------------------------------
intexEntertainment:
        call    spacepong                       ; run the space pong game
        jp      intexMainMenu                   ; return to main menu

;------------------------------------------------
; intexInfoBase - show the info base
;   input:  none
;   output: none
;------------------------------------------------
intexInfoBase:
#ifdef TI83PLUS
        ld      hl,intexInfoBaseData
#endif
#ifdef TI84PLUSCE
        ld      hl,(pDatFile)
        ld      de,intexInfoBaseData
        add     hl,de
#endif
        ld      b,(hl)                          ; B = number of pages of info base data
        inc     hl                              ; HL => actual text data
infoBaseLoop:
        push    bc                              ; save counter
        push    hl
#ifdef TI83PLUS
        call    textScreenBg
#endif
#ifdef TI84PLUSCE
        call    vramClear
#endif
        pop     hl
        call    displayPageNoClear
#ifdef TI83PLUS
        ld      a,(hl)                          ; A = # sprites for this page
        inc     hl                              ; HL => either sprite pointers or next page data
        or      a
        jr      z,infoBaseAfterSprites          ; if no sprites, skip this part
        ld      b,a                             ; B = sprite counter
infoBaseSprites:
        push    bc
        push    hl
        LDHLIND()                               ; HL => sprite coords followed by sprite data
        ld      b,(hl)                          ; B = y coord
        inc     hl
        ld      a,(hl)                          ; A = x coord
        inc     hl
#ifdef TI83PLUS
        push    bc
        call    spriteToRam                     ; copy sprite to ram, DE => this location
        pop     bc
#endif
#ifdef TI84PLUSCE
        ex      de,hl
#endif
        ld      l,b                             ; L = y coord
        bcall(ab5_putSprite)                    ; draw the sprite
        bcall(_grbufcpy)
        pop     hl
        pop     bc
        inc     hl
        inc     hl                              ; HL => next sprite pointer or next page data
        djnz    infoBaseSprites
infoBaseAfterSprites:
#endif
        pop     bc                              ; restore counter
infoBaseKeyLoop:
        push    bc                              ; save counter
        push    hl                              ; save data ptr
        call    waitKey
        pop     hl                              ; restore data ptr
        pop     bc                              ; restore counter
        cp      GK_MODE                         ; [MODE]?
        jr      z,infoBaseDone                  ; if so, info base is done
        cp      GK_2ND                          ; [2nd]?
        jr      nz,infoBaseKeyLoop              ; if not, wait for another key
        djnz    infoBaseLoop                    ; loop to next page
infoBaseDone:
        jp      intexMainMenu                   ; return to main menu

;------------------------------------------------
; intexAbort - leave the intex console
;   input:  none
;   output: none
;------------------------------------------------
intexAbort:
#ifdef TI83PLUS
        call    textScreenBg
#endif
#ifdef TI84PLUSCE
        call    vramClear
#endif
        ld      hl,strIntexAbort                ; HL => leaving intex text data
        call    displayPageNoClear
        ld      b,30                            ; B = wait count
        call    delay
#ifdef TI83PLUS
        bcall(ab5_restoreVideo)                 ; restore video buffer & leave
        ret
#endif
#ifdef TI84PLUSCE
        jp      fadeToGame
#endif

;------------------------------------------------
; intexBuy@WEAPON@ - try to buy said weapon
;   input:  none
;   output: none
;------------------------------------------------
intexBuyTwinLaser:
        ld      b,%00000010                     ; B = bit mask for weaponTable
        ld      de,CR_TWINLASER                 ; DE = cost
        jr      intexBuyWeapon
intexBuyFlameArc:
        ld      b,%00000100
        ld      de,CR_FLAMEARC
        jr      intexBuyWeapon
intexBuyPlasmaRifle:
        ld      b,%00001000
        ld      de,CR_PLASMARIFLE
        jr      intexBuyWeapon
intexBuyFlamethrower:
        ld      b,%00010000
        ld      de,CR_FLAMETHROWER
intexBuyWeapon:
        ld      a,(weaponTable)                 ; A = bit array of owned weapons
        and     b                               ; check if weapon already owned
        jp      nz,intexAlreadyOwned            ; if so, tell the player & return to main menu
        call    intexCheckCredits               ; check if enough credits available
        ld      hl,weaponTable                  ; HL => bit array of owned weapons
        ld      a,b                             ; A = bit mask to OR
        or      (hl)
        ld      (hl),a                          ; save new bit array of owned weapons
        ld      a,TIP_WEAPONS
        ld      d,1                             ; don't save gbuf in showTip
        call    showTip                         ; show weapons tip
        jr      intexItemPurchased

;------------------------------------------------
; intexBuy@ITEM@ - try to buy said item
;   input:  none
;   output: none
;------------------------------------------------
intexBuyKeyPack:
        ld      de,CR_KEYPACK                   ; DE = cost
        call    intexCheckCredits               ; check if enough credits available
        ld      hl,(pPlayer)
        ld      de,PK
        add     hl,de                           ; HL => key passes
        ld      a,6
        add     a,(hl)                          ; A = new number of key passes
        ld      (hl),a                          ; save it
        jr      intexItemPurchased

intexBuyAmmoClip:
        ld      de,CR_AMMOCLIP
        call    intexCheckCredits
        bcall(ab5_addAmmoClip)                   ; use the same routine as when player picks up a clip
        jr      intexItemPurchased

intexBuyFirstAid:
        ld      de,CR_FIRSTAID
        call    intexCheckCredits
        ld      hl,(pPlayer)                    ; HL => data for this player
        ld      de,PH
        add     hl,de                           ; HL => health of this player
        ld      a,MAX_HEALTH/2
        add     a,(hl)                          ; A = new health
        ld      (hl),a                          ; save it
        cp      MAX_HEALTH                      ; check for overflow
        jr      c,intexItemPurchased            ; if not, no need to cap it
        ld      (hl),MAX_HEALTH                 ; set max health
        jr      intexItemPurchased

intexBuyRadarScanner:
        ld      b,%00100000                     ; B = bit mask for weaponTable (radar scanner is stored as a 6th weapon)
        ld      de,CR_RADARSCANNER
        jr      intexBuyWeapon

intexBuyExtraLife:
        ld      de,CR_EXTRALIFE
        call    intexCheckCredits
        ld      hl,(pPlayer)
        ld      de,PL
        add     hl,de                           ; HL => player lives
        inc     (hl)                            ; inc lives
        ; fall through to intexPurchased

;------------------------------------------------
; intexPurchased - display a msg telling player that they purchased the item
;   input:  none
;   output: none
;------------------------------------------------
intexItemPurchased:
        ; check for all weapons/tools achievement
        ld      a,(aflagLongPlay)
        or      a
        jr      z,afterToolsAch
        ld      a,(weaponTable)
        xor     %00111111
        jr      nz,afterToolsAch
        ld      bc,ACH_ALLTOOLS_BIT*256+ACH_ALLTOOLS_ID
        ld      de,ACH_ALLTOOLS_OFFSET
        ld      hl,viewNewAchievement
        bcall(ab5_setAchievement)
afterToolsAch:
        ld      hl,strIntexPurchased            ; HL => string data
        jr      intexMsgWaitMenu                ; jump into the middle of intexAlreadyOwned

;------------------------------------------------
; intexCheckCredits - check if enough credits are available to buy an item
;   input:  DE = cost of item
;   output: none
;------------------------------------------------
intexCheckCredits:
        ld      hl,(credits)
        or      a                               ; clear carry
        sbc     hl,de                           ; HL = any left over credits, or negative if not enough
        jr      c,intexNotEnoughCredits         ; if not enough, tell player
        ld      (credits),hl                    ; save left over credits amount
        ret                                     ; ret to "intexBuy@@@@@" routine
intexNotEnoughCredits:
        pop     hl                              ; pop ret value from routine call
        ld      hl,strIntexNotEnoughCredits     ; HL => string data
        jr      intexMsgWaitMenu                ; jump into the middle of intexAlreadyOwned

;------------------------------------------------
; intexAlreadyOwned - display a msg telling player that they already own an item
;   input:  none
;   output: none
;------------------------------------------------
intexAlreadyOwned:
        ld      hl,strIntexAlreadyOwned         ; HL => string data
intexMsgWaitMenu:
        push    hl
#ifdef TI83PLUS
        call    textScreenBg
#endif
        pop     hl
        call    displayPageNoClear
        ld      b,50
        call    delay                           ; wait B times
        jp      intexMainMenu                   ; return to main menu

;------------------------------------------------
; viewRadarScanner - view radar scanner (if it is owned)
;   input:  none
;   output: none
;------------------------------------------------
viewRadarScanner:
        ld      hl,weaponTable                  ; HL => weapon flags (radar scanner is stored as a 6th weapon)
        bit     5,(hl)                          ; check if available
        ret     z                               ; if not, leave
#ifdef TI83PLUS
        bcall(ab5_saveVideo)                    ; save gbuf
        call    fadeOutAndClear
        call    textScreenBg
        ld      de,28*256+29
        ld      hl,strScanning
        call    vPuts                           ; "SCANNING..."
        bcall(_grbufcpy)    
#endif
#ifdef TI84PLUSCE
        call    fadeOutClear
#endif
        ld      b,80
        call    delay
        ld      a,(difficulty)
        or      a
        jr      z,viewRadarScannerShowMap       ; always show in NORMAL
        ; otherwise, difficulty is TOUGH
        ld      hl,levelFlags
        bit     LFLAG_COUNTDOWNON,(hl)          ; is countdown running
        jr      nz,viewRadarScannerDone         ; if so, can't show the map
viewRadarScannerShowMap:
        call    showMap                         ; show the level map
viewRadarScannerDone:
#ifdef TI83PLUS
        call    fadeOutAndClear
        bcall(ab5_restoreVideo)                 ; restore gbuf
#endif
#ifdef TI84PLUSCE
        call    fadeToGame
#endif
        ret

#ifdef TI83PLUS
;------------------------------------------------
; drawIntexCursor (83+) - draw the menu cursor using XOR
;   input:  none
;   output: none
;------------------------------------------------
drawIntexCursor:
        ld      hl,sprMenuItem
        call    spriteToRam
        ld      de,(intexMenuItem)
        ld      d,0                             ; DE = (intexMenuItem)
        ld      hl,(intexCursorTable)
        add     hl,de
        ld      a,(hl)                          ; A = cursor x
        ld      e,a                             ; E = cursor x
        ld      a,(intexMenuItem)
        inc     a                               ; A = menu item plus 1 (so that it can't = 0)
        ld      b,a
        ld      a,10+1-7                        ; A = cursor y for top option line minus 1
calcIntexCursorY:
        add     a,7                             ; A = next menu option line
        djnz    calcIntexCursorY
        ld      l,a                             ; L = y
        ld      a,e                             ; A = x
        ld      de,sprBuffer                    ; DE => sprite in ram
        bcall(ab5_putSprite)                    ; XOR sprite
        ret
#endif
#ifdef TI84PLUSCE
;------------------------------------------------
; drawIntexCursor (CE) - draw the menu cursor
;   input:  A = colour to use
;   output: none
;------------------------------------------------
drawIntexCursor:
        ld      (__dicColour),a
        ld      a,(intexMenuItem)
        ld      hl,0
        or      a
        jr      z,dicStart
        ld      de,15*320
        ld      b,a
dicCalc:
        add     hl,de
        djnz    dicCalc
dicStart:
        ld      de,75*320+48
        add     hl,de
        ld      de,(pVram)
        add     hl,de
        ld      de,320-7
        ld      c,11
dicOuterLoop:
        ld      b,7
dicInnerLoop:
__dicColour             = $+1
        ld      (hl),$00
        inc     hl
        djnz    dicInnerLoop
        add     hl,de
        dec     c
        jr      nz,dicOuterLoop
        ret
#endif

;------------------------------------------------
; showObjectives - show mission objectives
;   input:  none
;   output: HL => level header after mission objectives
;------------------------------------------------
showObjectives:
#ifdef TI83PLUS
        call    textScreenBg
#endif
#ifdef TI84PLUSCE
        call    vramClear
        ld      a,COLOUR_WHITE
        ld      (__dcColour),a
#endif
        ld      hl,(tempFile)                   ; HL => start of uncompressed level data, which is the mission objective text
        jp      displayPageNoClear              ; display page of text (without first clearing gbuf) and ret

;------------------------------------------------
; showEventText - show a page of text from an event
;   input:  HL => text data
;   output: none
;------------------------------------------------
showEventText:
#ifdef TI83PLUS
        push    hl
        call    textScreenBg
        pop     hl
        call    displayPageNoClear
        ld      hl,str2nd
        call    showStrings
        bcall(_grbufcpy)
        jp      wait2ndEnter
#endif
#ifdef TI84PLUSCE
        push    hl
        call    modifyText
        call    fadeOutClear
        pop     hl
        call    displayPage
        call    wait2ndEnter
        jp      fadeToGame
#endif

;------------------------------------------------
; textScreenBg - load the text screen background to gbuf
;   input:  none
;   output: none
;------------------------------------------------
#ifdef TI83PLUS
textScreenBg:
        bcall(_grbufclr)
        ld      hl,textBgConfig                 ; row layout data
        ld      ix,gbuf                         ; IX => start of gbuf
tsbgMain:
        ld      a,(hl)                          ; get byte of config data
        inc     a                               ; is it $FF?
        ret     z                               ; $FF means we're done
        ld      b,(hl)                          ; B = # times to display this row
        inc     hl
        ld de,0 \ ld e,(hl)                     ; DE = row type to show
        inc     hl                              ; HL => next config pair
        push    hl
        ex      de,hl                           ; HL = row type to show
        add     hl,hl                           ; x2
        add     hl,hl                           ; x4
        ld      e,l
        ld      d,h
        add     hl,hl                           ; x8
        add     hl,de                           ; x12
        ld      de,textBgRows
        add     hl,de                           ; HL => row data
tsbgRow:
        push    bc
        push    hl
        ld      b,12
tsbgRepeat:
        ld      a,(hl)                          ; A = byte of row data
        ld      (ix),a                          ; save to gbuf
        inc     hl
        inc     ix
        djnz    tsbgRepeat
        pop     hl                              ; HL => start of row data again (in case this row is to be displayed again)
        pop     bc                              ; B = row counter
        djnz    tsbgRow
        pop     hl                              ; HL => config data
        jr      tsbgMain
#endif

;------------------------------------------------
; displayChar - display a character to gbuf/vram and update pencol & penrow accordingly
;   input:  HL => character to display, NULL means line feed
;   output: HL => next character
;------------------------------------------------
displayChar:
        ld      a,(hl)                          ; A = character
        inc     hl                              ; HL => next character
        push    hl                              ; save character pointer
#ifdef TI83PLUS
        bcall(_vputmap)                         ; display character and update pencol & penrow accordingly
#endif
#ifdef TI84PLUSCE
        sub     $20
        ld      c,a
        ld      b,11
        mlt     bc
        ld      ix,chars
        add     ix,bc                           ; IX => char sprite
        ld      a,(penrow)
        ld      l,a
        ld      h,160
        mlt     hl
        add     hl,hl
        ld      a,(pencol)
        ld      e,a
        ld      d,2
        mlt     de
        add     hl,de
        ld      de,(pVram)
        add     hl,de                           ; HL => where on vram to draw character
        ld      bc,11*256+128                   ; B = 11 rows, C = %10000000 bitmask
        ld      de,320                          ; DE = 320 bytes of vram to skip per row (used at end of each row)
dcRow:
        push    hl
dcCol:
        ld a,(ix) \ and c                       ; check bit
        jr      z,dcNext
__dcColour              = $+1
        ld      (hl),$FF
dcNext:
        inc     hl
        rrc     c
        jr      nc,dcCol                        ; if bitmask hasn't fully rotated, do next column
        inc     ix                              ; IX => next byte of sprite
        pop     hl
        add     hl,de                           ; HL => where on vram to start drawing next row
        djnz    dcRow
        ld      a,(pencol)
        add     a,4
        ld      (pencol),a                      ; updated pencol
#endif
        pop     hl                              ; restore character pointer
        ret

;------------------------------------------------
; displayPage - display a page of text using the displayChar routine and update it to the lcd
;   input:  HL => page string data, each line NULL terminated, page is terminated by $FF
;   output: HL => data directly after this page's data
;   notes:  if [ALPHA] is pressed during the printing animation, it will skip out and write the rest of the text in one hit
;------------------------------------------------
displayPage:
        push    hl
#ifdef TI83PLUS
        bcall(_grbufclr)
#endif
#ifdef TI84PLUSCE
        call    vramClear
#endif
        pop     hl
displayPageNoClear:
#ifdef TI83PLUS
        res     apdable,(iy+apdflags)           ; disable apd
#endif
#ifdef TI84PLUSCE
        ld      a,OP_OR_A
        ld      (__dpDelay),a
#endif
displayPageLoop:
#ifdef TI83PLUS
        ld      d,(hl)                          ; D = new penrow
        inc     hl
        ld      e,(hl)                          ; E = new pencol
        inc     hl                              ; HL => next line string
        ld      (pencol),de                     ; set cursor for new line
#endif
#ifdef TI84PLUSCE
        ld      a,(hl)
        inc     hl
        ld      (penrow),a
        ld      a,(hl)
        inc     hl
        ld      (pencol),a
#endif
displayLineLoop:
#ifdef TI83PLUS
        ld      a,KG_1
        bcall(ab5_directIn)
        bit     DI_ALPHA,a                      ; [ALPHA]?
        jr      z,displayPageFast               ; if so, skip out of the animation loop
#endif
        ld      a,(hl)                          ; A = first character of line
        inc     a                               ; was it $FF
        jr      z,displayPageEndPage            ; if so, update ptr to next page data and ret
        dec     a                               ; was it $00
        jr      z,displayPageEndLine
        call    displayChar                     ; display char to gbuf
        push    hl                              ; save char ptr
#ifdef TI83PLUS
        bcall(_grbufcpy)
#endif
#ifdef TI84PLUSCE
        call    keyScan
        ld      a,(kbdG2)
        bit     kbitAlpha,a
        call    nz,displayPageFast
        ld      bc,2000
__dpDelay               = $
        or      a
        call    nc,delayBC
#endif
        pop     hl
        jr      displayLineLoop
displayPageEndLine:
        inc     hl                              ; HL => next line data
        jr      displayPageLoop                 ; load new pen coords & repeat
displayPageEndPage:
        inc     hl                              ; HL => data directly after this page's data
#ifdef TI83PLUS
        set     apdrunning,(iy+apdflags)        ; enable apd
#endif
        ret
#ifdef TI83PLUS
displayPageFast:
        call    vPutsNoCursor                   ; show the remainder of the current line without changing the cursor
displayFastLoop:
        dec     hl
        ld      a,(hl)                          ; A = last byte from previous string
        inc     a                               ; was it $FF?
        jr      z,displayFastEndPage            ; if so, we're done
        inc     hl                              ; HL => next string data
        ld      d,(hl)
        inc     hl
        ld      e,(hl)
        inc     hl
        call    vPuts
        jr      displayFastLoop
displayFastEndPage:
        push    hl
        bcall(_grbufcpy)                        ; update lcd to show all text
        pop     hl
        jr      displayPageEndPage
#endif
#ifdef TI84PLUSCE
displayPageFast:
        ld      a,OP_SCF
        ld      (__dpDelay),a
        ret
#endif

.end
