;===============================================================;
;                                                               ;
; Alien Breed 5                                                 ;
; In-game key press routines                                    ;
;                                                               ;
;===============================================================;

;------------------------------------------------
; checkKeys - check key presses based on 1 or 2 player game
;   input:  none
;   output: none
;------------------------------------------------
checkKeys:
        ld      a,(coop)
        or      a
        jp      nz,checkKeysCoop                ; if co-op game, different key handling
        ; fall through to checkKeysSingle

;------------------------------------------------
; checkKeysSingle - check keys for 1 player game
;   input:  none
;   output: none
;------------------------------------------------
        ld      hl,p1x
        ld      (pPlayer),hl                    ; set player pointer to player 1
        ; check if player was hurt last frame or dead. if either is true, player can't move this frame
        ld      hl,p1ht
        ld      a,(hl)
        or      a                               ; was player hurt last frame?
        ld      (hl),0
        jr      nz,afterPlayerMovement
        ld      a,(p1dd)
        or      a                               ; is player dead?
        ret     nz                              ; if so, skip key checks this frame

        ; check arrow key presses
        ld      ix,p1x                          ; IX => player 1 data
#ifdef TI83PLUS
        ld      a,KG_ARROW                      ; check key presses for arrow keys
        call    directIn
#endif
#ifdef TI84PLUSCE
        ld      a,(kbdG7)
#endif
        push    af
#ifdef TI83PLUS
        bit     DI_UP,a                         ; was [UP] pressed?
        call    z,playerUp                      ; if so, try to move player up
#endif
#ifdef TI84PLUSCE
        bit     kbitUp,a
        call    nz,playerUp
#endif
        pop     af
        push    af
#ifdef TI83PLUS
        bit     DI_DOWN,a                       ; was [DOWN] pressed?
        call    z,playerDown                    ; if so, try to move player down
#endif
#ifdef TI84PLUSCE
        bit     kbitDown,a
        call    nz,playerDown
#endif
        pop     af
        push    af
#ifdef TI83PLUS
        bit     DI_LEFT,a                       ; was [LEFT] pressed?
        call    z,playerLeft                    ; if so, try to move player left
#endif
#ifdef TI84PLUSCE
        bit     kbitLeft,a
        call    nz,playerLeft
#endif
        pop     af
#ifdef TI83PLUS
        bit     DI_RIGHT,a                      ; was [RIGHT] pressed?
        call    z,playerRight                   ; if so, try to move player right
#endif
#ifdef TI84PLUSCE
        bit     kbitRight,a
        call    nz,playerRight
#endif

afterPlayerMovement:
        ; check other key presses
#ifdef TI83PLUS
        ld      a,KG_1                          ; check key presses for 1st column
        call    directIn
#endif
#ifdef TI84PLUSCE
        ld      a,(kbdG2)
#endif
#ifdef TI83PLUS
        push    af
        bit     DI_MATH,a                       ; [MATH]?
        bcallz(ab5_teacherKey)                  ; if so, turn off calc
        pop     af
#endif
#ifdef TI83PLUS
        bit     DI_ALPHA,a                      ; [ALPHA]?
        call    z,playerUse                     ; if so, try to use something in front of player
#endif
#ifdef TI84PLUSCE
        bit     kbitAlpha,a
        call    nz,playerUse
#endif
        
#ifdef TI83PLUS
        ld      a,KG_5
        call    directIn
#endif
#ifdef TI84PLUSCE
        ld      a,(kbdG6)
#endif
#ifdef TI83PLUS
        bit     DI_CLEAR,a                      ; [CLEAR]?
        bcallz(ab5_saveState)                   ; if so, try to save state and quit
#endif
#ifdef TI84PLUSCE
        bit     kbitClear,a
        call    nz,saveState
#endif
        
        ; check level skipping cheat
        ld      hl,e2cheatFlags
#ifdef LEVEL_SKIP
        set     skipLevelCheat,(hl)
#endif
        bit     skipLevelCheat,(hl)
        jr      z,afterSkipLevelCheat
#ifdef TI83PLUS
        ld      a,KG_3
        call    directIn
#endif
#ifdef TI84PLUSCE
        ld      a,(kbdG4)
#endif
#ifdef TI83PLUS
        bit     DI_DEC,a                        ; [.]?
        ld      ix,aFinish
        bcallz(ab5_levelFinished)
#endif
#ifdef TI84PLUSCE
        bit     kbitDecPnt,a
        ld      ix,aFinish
        call    nz,levelFinishedCheat
#endif
afterSkipLevelCheat:

#ifdef TI83PLUS
        ld      a,KG_TOP
        call    directIn                        ; check key presses for top row
#endif
#ifdef TI84PLUSCE
        ld      a,(kbdG1)
#endif
        push    af
#ifdef TI83PLUS
        bit     DI_Y,a                          ; [Y=]?
        call    z,selectWeapon1                 ; if so, select machine gun
#endif
#ifdef TI84PLUSCE
        bit     kbitYequ,a
        call    nz,selectWeapon1
#endif
        pop     af
        push    af
#ifdef TI83PLUS
        bit     DI_WINDOW,a                     ; [WINDOW]?
        call    z,selectWeapon2                 ; if so, select twin laser
#endif
#ifdef TI84PLUSCE
        bit     kbitWindow,a
        call    nz,selectWeapon2
#endif
        pop     af
        push    af
#ifdef TI83PLUS
        bit     DI_ZOOM,a                       ; [ZOOM]?
        call    z,selectWeapon3                 ; if so, select flame arc
#endif
#ifdef TI84PLUSCE
        bit     kbitZoom,a
        call    nz,selectWeapon3
#endif
        pop     af
        push    af
#ifdef TI83PLUS
        bit     DI_TRACE,a                      ; [TRACE]?
        call    z,selectWeapon4                 ; if so, select plasma rifle
#endif
#ifdef TI84PLUSCE
        bit     kbitTrace,a
        call    nz,selectWeapon4
#endif
        pop     af
        push    af
#ifdef TI83PLUS
        bit     DI_GRAPH,a                      ; [GRAPH]?
        call    z,selectWeapon5                 ; if so, select flamethrower
#endif
#ifdef TI84PLUSCE
        bit     kbitGraph,a
        call    nz,selectWeapon5
#endif
        pop     af
        push    af
#ifdef TI83PLUS
        bit     DI_2ND,a                        ; [2nd]?
        ld      a,P1_BLAST
        call    z,playerShoot                   ; if so, try to shoot
#endif
#ifdef TI84PLUSCE
        bit     kbit2nd,a
        ld      a,P1_BLAST
        call    nz,playerShoot
#endif
        pop     af
        push    af
#ifdef TI83PLUS
        bit     DI_DEL,a                        ; [DEL]?
        bcallz(ab5_viewRadarScanner)            ; if so, view it
#endif
#ifdef TI84PLUSCE
        bit     kbitDel,a
        call    nz,viewRadarScanner
#endif
        pop     af
#ifdef TI83PLUS
        bit     DI_MODE,a                       ; [MODE]?
        bcallz(ab5_pauseScreen)                 ; if so, pause
#endif
#ifdef TI84PLUSCE
        bit     kbitMode,a
        call    nz,pauseScreen
#endif
        ret

;------------------------------------------------
; checkKeysCoop - check keys for 2 player game
;   input:  none
;   output: none
;------------------------------------------------
checkKeysCoop:
        ; handle player 1 specific keys
        ld      hl,p1x
        ld      (pPlayer),hl                    ; point to player 1 data
        xor     a
        ld      (player),a
        ld      a,(p1p)
        or      a
        jp      z,afterP1Keys                   ; if player no longer playing/active, skip
        ld      hl,p1ht
        ld      a,(hl)
        or      a                               ; was player hurt last frame?
        ld      (hl),0
        jr      nz,afterP1Movement
        ld      a,(p1dd)
        or      a                               ; is player dead?
        jr      nz,afterP1Keys                  ; if so, skip key checks this frame
        
        ; check player 1 movement
        ld      ix,p1x                          ; IX => player 1 data
#ifdef TI83PLUS
        ld      a,KG_TOP
        call    directIn
#endif
#ifdef TI84PLUSCE
        ld      a,(kbdG1)
#endif
#ifdef TI83PLUS
        bit     DI_2ND,a
        call    z,playerUp
#endif
#ifdef TI84PLUSCE
        bit     kbit2nd,a
        call    nz,playerUp
#endif
#ifdef TI83PLUS
        ld      a,KG_1
        call    directIn
#endif
#ifdef TI84PLUSCE
        ld      a,(kbdG2)
#endif
        push    af
#ifdef TI83PLUS
        bit     DI_MATH,a
        call    z,playerDown
#endif
#ifdef TI84PLUSCE
        bit     kbitMath,a
        call    nz,playerDown
#endif
        pop     af
#ifdef TI83PLUS
        bit     DI_ALPHA,a
        call    z,playerLeft
#endif
#ifdef TI84PLUSCE
        bit     kbitAlpha,a
        call    nz,playerLeft
#endif
#ifdef TI83PLUS
        ld      a,KG_2
        call    directIn
#endif
#ifdef TI84PLUSCE
        ld      a,(kbdG3)
#endif
#ifdef TI83PLUS
        bit     DI_XTON,a
        call    z,playerRight
#endif
#ifdef TI84PLUSCE
        bit     kbitGraphVar,a
        call    nz,playerRight
#endif
        
afterP1Movement:
        ; check player 1 shoot/use
#ifdef TI83PLUS
        ld      a,KG_TOP
        call    directIn
#endif
#ifdef TI84PLUSCE
        ld      a,(kbdG1)
#endif
        push    af
#ifdef TI83PLUS
        bit     DI_WINDOW,a                     ; [WINDOW]?
        ld      a,P1_BLAST
        call    z,playerShoot
#endif
#ifdef TI84PLUSCE
        bit     kbitWindow,a
        ld      a,P1_BLAST
        call    nz,playerShoot
#endif
        pop     af
#ifdef TI83PLUS
        bit     DI_Y,a                          ; [Y=]?
        call    z,playerUse
#endif
#ifdef TI84PLUSCE
        bit     kbitYequ,a
        call    nz,playerUse
#endif
        ; check player 1 weapon selection
#ifdef TI83PLUS
        ld      a,KG_1
        call    directin
#endif
#ifdef TI84PLUSCE
        ld      a,(kbdG2)
#endif
        push    af
#ifdef TI83PLUS
        bit     DI_X1,a                         ; [X^-1]?
        call    z,selectWeapon1
#endif
#ifdef TI84PLUSCE
        bit     kbitRecip,a
        call    nz,selectWeapon1
#endif
        pop     af
        push    af
#ifdef TI83PLUS
        bit     DI_X2,a                         ; [X^2]?
        call    z,selectWeapon2
#endif
#ifdef TI84PLUSCE
        bit     kbitSquare,a
        call    nz,selectWeapon2
#endif
        pop     af
        push    af
#ifdef TI83PLUS
        bit     DI_LOG,a                        ; [LOG]?
        call    z,selectWeapon3
#endif
#ifdef TI84PLUSCE
        bit     kbitLog,a
        call    nz,selectWeapon3
#endif
        pop     af
        push    af
#ifdef TI83PLUS
        bit     DI_LN,a                         ; [LN]?
        call    z,selectWeapon4
#endif
#ifdef TI84PLUSCE
        bit     kbitLn,a
        call    nz,selectWeapon4
#endif
        pop     af
#ifdef TI83PLUS
        bit     DI_STO,a                        ; [STO>]?
        call    z,selectWeapon5
#endif
#ifdef TI84PLUSCE
        bit     kbitStore,a
        call    nz,selectWeapon5
#endif

afterP1Keys:
        ; handle player 2 specific keys
        ld      hl,p2x
        ld      (pPlayer),hl
        ld      a,1
        ld      (player),a
        ld      a,(p2p)
        or      a
        jp      z,afterP2Keys
        ld      hl,p2ht
        ld      a,(hl)
        or      a                               ; was player hurt last frame?
        ld      (hl),0
        jr      nz,afterP2Movement
        ld      a,(p2dd)
        or      a                               ; is player dead?
        jr      nz,afterP2Keys                  ; if so, skip key checks this frame
        
        ; check player 2 movement
        ld      ix,p2x                          ; IX => player 2 data
#ifdef TI83PLUS
        ld      a,KG_ARROW                      ; check key presses for arrow keys
        call    directIn
#endif
#ifdef TI84PLUSCE
        ld      a,(kbdG7)
#endif
        push    af
#ifdef TI83PLUS
        bit     DI_UP,a                         ; was [UP] pressed?
        call    z,playerUp                      ; if so, try to move player up
#endif
#ifdef TI84PLUSCE
        bit     kbitUp,a
        call    nz,playerUp
#endif
        pop     af
        push    af
#ifdef TI83PLUS
        bit     DI_DOWN,a                       ; was [DOWN] pressed?
        call    z,playerDown                    ; if so, try to move player down
#endif
#ifdef TI84PLUSCE
        bit     kbitDown,a
        call    nz,playerDown
#endif
        pop     af
        push    af
#ifdef TI83PLUS
        bit     DI_LEFT,a                       ; was [LEFT] pressed?
        call    z,playerLeft                    ; if so, try to move player left
#endif
#ifdef TI84PLUSCE
        bit     kbitLeft,a
        call    nz,playerLeft
#endif
        pop     af
#ifdef TI83PLUS
        bit     DI_RIGHT,a                      ; was [RIGHT] pressed?
        call    z,playerRight                   ; if so, try to move player right
#endif
#ifdef TI84PLUSCE
        bit     kbitRight,a
        call    nz,playerRight
#endif
        
afterP2Movement:
        ; check player 2 shoot/use
#ifdef TI83PLUS
        ld      a,KG_4
        call    directIn
#endif
#ifdef TI84PLUSCE
        ld      a,(kbdG5)
#endif
#ifdef TI83PLUS
        bit     DI_TAN,a
        ld      a,P2_BLAST
        call    z,playerShoot
#endif
#ifdef TI84PLUSCE
        bit     kbitTan,a
        ld      a,P2_BLAST
        call    nz,playerShoot
#endif
#ifdef TI83PLUS
        ld      a,KG_5
        call    directIn
#endif
#ifdef TI84PLUSCE
        ld      a,(kbdG6)
#endif
#ifdef TI83PLUS
        bit     DI_POWER,a
        call    z,playerUse
#endif
#ifdef TI84PLUSCE
        bit     kbitPower,a
        call    nz,playerUse
#endif
        ; check player 2 weapon selection
#ifdef TI83PLUS
        ld      a,KG_5
        call    directin
#endif
#ifdef TI84PLUSCE
        ld      a,(kbdG6)
#endif
        push    af
#ifdef TI83PLUS
        bit     DI_ENTER,a                      ; [ENTER]?
        call    z,selectWeapon1
#endif
#ifdef TI84PLUSCE
        bit     kbitEnter,a
        call    nz,selectWeapon1
#endif
        pop     af
        push    af
#ifdef TI83PLUS
        bit     DI_PLUS,a                       ; [+]?
        call    z,selectWeapon2
#endif
#ifdef TI84PLUSCE
        bit     kbitAdd,a
        call    nz,selectWeapon2
#endif
        pop     af
        push    af
#ifdef TI83PLUS
        bit     DI_MINUS,a                      ; [-]?
        call    z,selectWeapon3
#endif
#ifdef TI84PLUSCE
        bit     kbitSub,a
        call    nz,selectWeapon3
#endif
        pop     af
        push    af
#ifdef TI83PLUS
        bit     DI_MUL,a                        ; [x]?
        call    z,selectWeapon4
#endif
#ifdef TI84PLUSCE
        bit     kbitMul,a
        call    nz,selectWeapon4
#endif
        pop     af
#ifdef TI83PLUS
        bit     DI_DIV,a                        ; [/]?
        call    z,selectWeapon5
#endif
#ifdef TI84PLUSCE
        bit     kbitDiv,a
        call    nz,selectWeapon5
#endif

afterP2Keys:
#ifdef COOP_SYNC
#ifdef TI83PLUS
        ld      a,KG_2
        call    directIn
        bit     DI_1,a
        jr      nz,afterSyncP1
#endif
#ifdef TI84PLUSCE
        ld      a,(kbdG3)
        bit     kbit1,a
        jr      z,afterSyncP1
#endif
        ld      hl,(p1x)
        ld      (p2x),hl
        ld      hl,(p1y)
        ld      (p2y),hl
        ld      hl,(p1scry)
        ld      (p2scry),hl
afterSyncP1:
#ifdef TI83PLUS
        ld      a,KG_3
        call    directIn
        bit     DI_2,a
        jr      nz,afterSyncP2
#endif
#ifdef TI84PLUSCE
        ld      a,(kbdG4)
        bit     kbit2,a
        jr      z,afterSyncP2
#endif
        ld      hl,(p2x)
        ld      (p1x),hl
        ld      hl,(p2y)
        ld      (p1y),hl
        ld      hl,(p2scry)
        ld      (p1scry),hl
afterSyncP2:
#endif

        ; check other misc keys
#ifdef TI83PLUS
        ld      a,KG_TOP
        call    directin
#endif
#ifdef TI84PLUSCE
        ld      a,(kbdG1)
#endif
        push    af
#ifdef TI83PLUS
        bit     DI_DEL,a                        ; [DEL]?
        bcallz(ab5_viewRadarScanner)
#endif
#ifdef TI84PLUSCE
        bit     kbitDel,a
        call    nz,viewRadarScanner
#endif
        pop     af
#ifdef TI83PLUS
        bit     DI_MODE,a                       ; [MODE]?
        bcallz(ab5_pauseScreen)
#endif
#ifdef TI84PLUSCE
        bit     kbitMode,a
        call    nz,pauseScreen
#endif
        ; check quick save and quit
#ifdef TI83PLUS
        ld      a,KG_3
        call    directIn
#endif
#ifdef TI84PLUSCE
        ld      a,(kbdG4)
#endif
#ifdef TI83PLUS
        bit     DI_PRGM,a                       ; [PRGM]?
        bcallz(ab5_saveState)                   ; if so, try to save state and quit
#endif
#ifdef TI84PLUSCE
        bit     kbitPgrm,a
        call    nz,saveState
#endif
        ; check level skipping cheat
        ld      hl,e2cheatFlags
#ifdef LEVEL_SKIP
        set     skipLevelCheat,(hl)
#endif
        bit     skipLevelCheat,(hl)
        ret     z
#ifdef TI83PLUS
        ld      a,KG_3
        call    directIn
#endif
#ifdef TI84PLUSCE
        ld      a,(kbdG4)
#endif
#ifdef TI83PLUS
        bit     DI_DEC,a                        ; [.]?
        ld      ix,aFinish
        bcallz(ab5_levelFinished)
#endif
#ifdef TI84PLUSCE
        bit     kbitDecPnt,a
        ld      ix,aFinish
        call    nz,levelFinishedCheat
#endif
        ret

.end
