;===============================================================;
;                                                               ;
; Alien Breed 5                                                 ;
; Miscellaneous routines                                        ;
;                                                               ;
;===============================================================;

#ifdef TI83PLUS
;------------------------------------------------
; saveVideo - save gbuf to videoBuffer
;   input:  none
;   output: none
;------------------------------------------------
saveVideo:
        ld      hl,gbuf
        ld      de,videoBuffer
copyVideo:
        ld      bc,768
        ldir
        ret

;------------------------------------------------
; restoreVideo - restore videoBuffer to gbuf
;   input:  none
;   output: none
;------------------------------------------------
restoreVideo:
        ld      hl,videoBuffer
        ld      de,gbuf
        jr      copyVideo
#endif

;------------------------------------------------
; absHL - check if HL is negative, and if so, make it positive
;   input:  HL = value to check
;   output: HL = abs(HL)
;------------------------------------------------
absHL:
        bit     7,h                             ; check if MSB is set
        ret     z                               ; if not, HL is a positive value
#ifdef TI83PLUS
        ld      a,h
        cpl
        ld      h,a
        ld      a,l
        cpl
        ld      l,a
        inc     hl                              ; 2's complement of HL
#endif
#ifdef TI84PLUSCE
        ex      de,hl
        call    _NegDE
        ex      de,hl
#endif
        ret

;------------------------------------------------
; average - calculate the average between two values
;   input:  HL = value 1
;           DE = value 2
;   output: HL = average
;------------------------------------------------
average:
        CPHLDE()                                ; check if DE is greater than HL
        jr      nc,averageNoSwap                ; if not, no need to swap
        ex      de,hl
averageNoSwap:
        or      a                               ; clear carry
        sbc     hl,de                           ; HL = difference (will be less than 256 as players are always within a screen length of each other)
        srl     l                               ; HL = difference/2
        add     hl,de                           ; HL = lesser value + (difference/2), ie. the average
        ret

#ifdef TI83PLUS
;------------------------------------------------
; directIn - read keypresses from a key group
;   input:  A = key group code (83+/84+)
;           HL = keypad register address (84+CE)
;   output: A = key presses from specified group
;------------------------------------------------
directIn:
        ld      bc,$FF01
        out     (c),b                           ; clear key port
        out     (c),a                           ; open specified key group
        in      a,(c)                           ; A = key flags
        out     (c),b                           ; clear key port again
        ret
#endif

#ifdef TI84PLUSCE
;------------------------------------------------
; keyScan - perform a keyscan
;   input:  none
;   output: none
;------------------------------------------------
keyScan:
        di
        ld      hl,DI_Mode
        ld      (hl),$02
        xor     a
ksWait:
        cp      (hl)
        jr      nz,ksWait
        ret
#endif

;------------------------------------------------
; getRandom - generate a random number
;   input:  B = upper bound
;   output: A = result 0<=A<B
;   notes:  taken from Ion code, credit goes to JoeW
;           DE & HL preserved
;------------------------------------------------
getRandom:
        push    hl
        push    de
        ld      hl,(randData)
        ld      a,r
        ld      d,a
        ld      e,(hl)
        add     hl,de
        add     a,l
        xor     h
        ld      (randData),hl
        sbc     hl,hl
        ld      e,a
        ld      d,h
randomLoop:
        add     hl,de
        djnz    randomLoop
        ld      a,h
        pop     de
        pop     hl
        ret

.end
