;===============================================================;
;                                                               ;
; Alien Breed 5                                                 ;
; Space pong routines                                           ;
;                                                               ;
;===============================================================;

;------------------------------------------------
; spacepong - play space pong!
;   input:  none
;   output: none
;------------------------------------------------
spacepong:
#ifdef TI83PLUS
        res     apdable,(iy+apdflags)           ; disable apd during pong
#endif
        ld      b,INTO_GAME_DELAY
        call    delay                           ; tiny delay to avoid serving as soon as starting the match
#ifdef TI83PLUS
        call    drawPongField                   ; first draw the play screen
#endif
#ifdef TI84PLUSCE
        call    vramClear
#endif
        ; reset all positions
        ld      a,PONG_PADY
        ld      (pong1),a                       ; set player 1 paddle
        ld      a,PONG_LEFT
        ld      (pongBallx),a                   ; set ball x coord
        ld      hl,0
        ld      (pong1score),hl                 ; reset both scores
        ld      hl,$0101
        ld      (pongMovey),hl                  ; set initial ball movement
pongNewPoint:
#ifdef TI83PLUS
        call    pongShowScore                   ; update the score to gbuf/vbuf's
#endif
        xor     a
        ld      (pongMode),a                    ; reset mode to "new point"
        ld      (pongRallyHits),a               ; reset rally length
        ld      a,(pong1)
        ld      (pong2),a                       ; set player 2 paddle to same position as player 1
        add     a,2
        ld      (pongBally),a                   ; set ball y coord to slightly above centre of player's paddle
pongLoop:
        ld      hl,pongFrame
        inc     (hl)                            ; update frame counter

        call    pongKeys
        ld      a,(coop)
        or      a
        call    z,pongCalcMove                  ; if not a co-op game, the calc is player 2
        call    pongBall                        ; move the ball
        
#ifdef TI83PLUS
        call    xorPongSprites                  ; draw sprites
        bcall(_grbufcpy)                        ; gbuf to lcd
        call    xorPongSprites                  ; erase sprites
#endif
#ifdef TI84PLUSCE
        call    drawPongField
        call    vramFlip
#endif
        
        jr      pongLoop

;------------------------------------------------
; pongKeys - handle key presses
;   input:  none
;   output: none
;------------------------------------------------
pongKeys:
#ifdef TI84PLUSCE
        call    keyScan
#endif
        ld      a,(coop)
        or      a
        jr      nz,pong2Pkeys
        
pong1Pkeys:
        ld      d,0                             ; player id
#ifdef TI83PLUS
        ld      a,KG_TOP
        call    pongKeysIn
#endif
#ifdef TI84PLUSCE
        ld      a,(kbdG1)
#endif
#ifdef PONG_CHEAT
        push    af
#ifdef TI83PLUS
        bit     DI_DEL,a
        jr      z,pongCheat
#endif
#ifdef TI84PLUSCE
        bit     kbitDel,a
        jr      nz,pongCheat
#endif
        pop     af
#endif
        push    af
#ifdef TI83PLUS
        bit     DI_2ND,a                        ; [2nd]
        call    z,pongServe
#endif
#ifdef TI84PLUSCE
        bit     kbit2nd,a
        call    nz,pongServe
#endif
        pop     af
#ifdef TI83PLUS
        bit     DI_MODE,a                       ; [MODE]
        jr      z,pongDone
#endif
#ifdef TI84PLUSCE
        bit     kbitMode,a
        jr      nz,pongDone
#endif
        
#ifdef TI83PLUS
        ld      a,KG_ARROW
        call    pongKeysIn
#endif
#ifdef TI84PLUSCE
        ld      a,(kbdG7)
#endif
        ld      hl,pong1                        ; used in pongUp / pongDown
        push    af
#ifdef TI83PLUS
        bit     DI_UP,a                         ; [Up]
        call    z,pongUp
#endif
#ifdef TI84PLUSCE
        bit     kbitUp,a
        call    nz,pongUp
#endif
        pop     af
#ifdef TI83PLUS
        bit     DI_DOWN,a                       ; [Down]
        call    z,pongDown
#endif
#ifdef TI84PLUSCE
        bit     kbitDown,a
        call    nz,pongDown
#endif
        ret

pong2Pkeys:
        ; player 1 keys in co-op game
        ld      d,0
#ifdef TI83PLUS
        ld      a,KG_TOP
        call    pongKeysIn
#endif
#ifdef TI84PLUSCE
        ld      a,(kbdG1)
#endif
        push    af
#ifdef TI83PLUS
        bit     DI_WINDOW,a                     ; [WINDOW]
        call    z,pongServe
#endif
#ifdef TI84PLUSCE
        bit     kbitWindow,a
        call    nz,pongServe
#endif
        pop     af
        push    af
        ld      hl,pong1
#ifdef TI83PLUS
        bit     DI_2ND,a                        ; [2nd]
        call    z,pongUp
#endif
#ifdef TI84PLUSCE
        bit     kbit2nd,a
        call    nz,pongUp
#endif
        pop     af
#ifdef TI83PLUS
        bit     DI_MODE,a                       ; [MODE]
        jr      z,pongDone
#endif
#ifdef TI84PLUSCE
        bit     kbitMode,a
        jr      nz,pongDone
#endif
        
#ifdef TI83PLUS
        ld      a,KG_1
        call    pongKeysIn
        bit     DI_MATH,a                       ; [MATH]
        call    z,pongDown
#endif
#ifdef TI84PLUSCE
        ld      a,(kbdG2)
        bit     kbitMath,a
        call    nz,pongDown
#endif
        
        ; player 2 keys in co-op game
        ld      d,1
#ifdef TI83PLUS
        ld      a,KG_4
        call    pongKeysIn
        bit     DI_TAN,a                        ; [TAN]
        call    z,pongServe
#endif
#ifdef TI84PLUSCE
        ld      a,(kbdG5)
        bit     kbitTan,a
        call    nz,pongServe
#endif
        
#ifdef TI83PLUS
        ld      a,KG_ARROW
        call    pongKeysIn
#endif
#ifdef TI84PLUSCE
        ld      a,(kbdG7)
#endif
        push    af
        ld      hl,pong2
#ifdef TI83PLUS
        bit     DI_UP,a                         ; [Up]
        call    z,pongUp
#endif
#ifdef TI84PLUSCE
        bit     kbitUp,a
        call    nz,pongUp
#endif
        pop     af
#ifdef TI83PLUS
        bit     DI_DOWN,a                       ; [Down]
        call    z,pongDown
#endif
#ifdef TI84PLUSCE
        bit     kbitDown,a
        call    nz,pongDown
#endif
        ret

pongDone:
        pop     hl                              ; clear stack
        ret

#ifdef PONG_CHEAT
pongCheat:
        pop     af
        ld      a,PONG_RIGHT-1
        ld      (pongBallx),a
        ld      a,(pongBally)
        add     a,8
        ld      (pong2),a
        ld      a,1
        ld      (pongMovex),a
        ret
#endif

#ifdef TI83PLUS
;------------------------------------------------
; pongKeysIn - duplicate version of directIn routine used during main game (to save ROM page swapping every frame)
;   input:  A = key group code
;   output: A = key flags
;------------------------------------------------
pongKeysIn:
        ld      bc,$FF01
        out     (c),b
        out     (c),a
        in      a,(c)
        ret
#endif

;------------------------------------------------
; pongUp - try to move paddle up (either player)
;   input:  HL => paddle y coord
;           D = player id (0,1)
;   output: HL => paddle y coord
;           D = player id (0,1)
;------------------------------------------------
pongUp:
        ld      a,(hl)
        cp      2
        ret     z
        dec     (hl)
        ; if a new point and a single player game, move the calc paddle as well
        ld      a,(pongMode)
        or      a
        ret     nz                              ; not a new point
        push    hl
        ld      a,(coop)
        or      a
        jr      nz,pongUpCoop                   ; co-op game, so don't move other paddle
        inc     hl                              ; HL => calc paddle
        dec     (hl)
        jr      pongUpBall
pongUpCoop:
        ld      a,d
        or      a
        ld      a,(pongBallx)
        jr      nz,pongUpP2
pongUpP1:
        cp      PONG_LEFT
        jr      z,pongUpBall
        jr      pongUpDone
pongUpP2:
        cp      PONG_RIGHT
        jr      nz,pongUpDone
pongUpBall:
        ld      hl,pongBally
        dec     (hl)
pongUpDone:
        pop     hl
        ret

;------------------------------------------------
; pongDown - try to move paddle down (either player)
;   input:  HL => paddle y coord
;           D = player id (0,1)
;   output: HL => paddle y coord
;           D = player id (0,1)
;------------------------------------------------
pongDown:
        ld      a,(hl)
        cp      64-16-2
        ret     z
        inc     (hl)
        ; if a new point and a single player game, move the calc paddle as well
        ld      a,(pongMode)
        or      a
        ret     nz
        push    hl
        ld      a,(coop)
        or      a
        jr      nz,pongDownCoop
        inc     hl
        inc     (hl)
        jr      pongDownBall
pongDownCoop:
        ld      a,d
        or      a
        ld      a,(pongBallx)
        jr      nz,pongDownP2
pongDownP1:
        cp      PONG_LEFT
        jr      z,pongDownBall
        jr      pongDownDone
pongDownP2:
        cp      PONG_RIGHT
        jr      nz,pongDownDone
pongDownBall:
        ld      hl,pongBally
        inc     (hl)
pongDownDone:
        pop     hl
        ret

;------------------------------------------------
; pongCalcMove - move the calc paddle
;   input:  HL => paddle position
;   output: none
;------------------------------------------------
pongCalcMove:
        ld      a,(pongMode)
        or      a
        ret     z                               ; if start of a point then leave
        push    hl
        ld      a,(pongBallx)
        ld      hl,pongCalcFollow
        cp      (hl)                            ; has ball moved past the calc "reaction" point?
        ld      a,(pongFrame)
        ld      b,%00000011
        jr      c,pcmFrameCheck                 ; if not, move every 4th frame
        srl     b                               ; otherwise move every 2nd frame
pcmFrameCheck:
        and     b
        pop     hl
        ret     nz
        ld      hl,pong2
        ld      a,(pongBally)
        sub     3
        cp      (hl)
        jp      c,pongUp
        jr      pongDown

;------------------------------------------------
; pongServe - serve the ball (only if it's the start of a new point)
;   input:  D = player id (0,1)
;   output: D = player id (0,1)
;------------------------------------------------
pongServe:
        ld      a,(pongMode)
        or      a
        ret     nz                              ; if not a new point, leave
        ld      a,(coop)
        or      a
        jr      z,pongServeGo
        ld      a,d
        or      a
        ld      a,(pongBallx)
        jr      nz,pongServe2
pongServe1:
        cp      PONG_LEFT
        jr      z,pongServeGo
        ret
pongServe2:
        cp      PONG_RIGHT
        ret     nz
pongServeGo:
        ld      a,1
        ld      (pongMode),a                    ; set mode to rallying
        ret

;------------------------------------------------
; pongBall - move the ball (unless in serving mode)
;   input:  none
;   output: none
;------------------------------------------------
pongBall:
        ld      a,(pongMode)
        or      a
        ret     z                               ; if it's a new point (serving mode), leave
        ld      a,(pongFrame)
        bit     0,a
        ret     z                               ; only move every 2nd frame
        ld      hl,pongBally
        ld      a,(pongMovey)
        add     a,(hl)
        ld      (hl),a                          ; save new y coord
        cp      PONG_BOTTOM                     ; hit bottom of play field?
        jr      nz,pbCheckTop                   ; if not, check for top of play field
        ld      a,-1
        jr      pbSaveMovey
pbCheckTop:
        cp      PONG_TOP                        ; hit top of play field?
        jr      nz,pbMoveHoriz                  ; if not, go to horizontal movement
        ld      a,1
pbSaveMovey:
        ld      (pongMovey),a                   ; save new value
pbMoveHoriz:
        inc     hl                              ; HL => ball x coord
        ld      a,(pongMovex)
        add     a,(hl)
        ld      (hl),a                          ; save new x coord
        dec     hl                              ; HL => pongBally, will be used if the ball has hit a screen edge
        cp      PONG_RIGHT                      ; hit right edge of play field?
        jr      z,pbCheckP2Hit                  ; if so, check if P2 paddle is there
pbCheckLeft:
        cp      PONG_LEFT                       ; hit left edge of play field?
        ret     nz                              ; if not, leave
pbCheckP1Hit:
        push    hl
        ld      hl,pongRallyHits
        inc     (hl)                            ; increment player hit counter
        ld      b,PONG_FOLLOWMAX-PONG_FOLLOWMIN
        bcall(ab5_getRandom)
        add     a,PONG_FOLLOWMIN
        ld      (pongCalcFollow),a              ; save new x coord for calc paddle speed
        pop     hl
        ld      a,(pong1)
        sub     2
        cp      (hl)
        jr      nc,pbP1Missed
        add     a,10
        cp      (hl)
        jr      nc,pbBallRight
pbP1Missed:
        xor     a
        ld      (pongRallyHits),a
        ld      hl,pong2score
        inc     (hl)
        ld      a,(hl)
        cp      PONG_WIN                        ; has p2 won?
        ld      a,1                             ; set status
        jr      nc,pbGameOver
        pop     hl                              ; clear RET
        ld      hl,pongNewPoint
        push    hl                              ; load new RET addr
pbBallRight:
        ld      a,1
        jr      pbSaveMovex
pbCheckP2Hit:
        ld      a,(pong2)
        sub     2
        cp      (hl)
        jr      nc,pbP2Missed
        add     a,10
        cp      (hl)
        jr      nc,pbBallLeft
pbP2Missed:
        xor     a
        ld      (pongRallyHits),a
        ; set the achievement if not a co-op game
        ld      a,(coop)
        or      a
        jr      nz,pbAfterPointAch
        ld      bc,ACH_PLAYPONG_BIT*256+ACH_PLAYPONG_ID
        ld      de,ACH_PLAYPONG_OFFSET
        ld      hl,viewNewAchievement
        bcall(ab5_setAchievement)
        call    drawPongField                   ; need to re-draw the play field
pbAfterPointAch:
        ; increase score, etc.
        ld      hl,pong1score
        inc     (hl)
        ld      a,(hl)
        cp      PONG_WIN                        ; has p1 won?
        ld      a,0                             ; set status (can't use xor a as it will corrupt the NC/CA result)
        jr      nc,pbGameOver
        pop     hl
        ld      hl,pongNewPoint
        push    hl
pbBallLeft:
        ld      a,-1
pbSaveMovex:
        ld      b,a                             ; B = x movement
        ld      a,(pongRallyHits)
        cp      PONG_FAST                       ; check # player hits
        ld      a,b                             ; A = x movement
        jr      c,pbDone
        add     a,a                             ; if rally has gone on long enough, double the x speed
pbDone:
        ld      (pongMovex),a
        ret
pbGameOver:
        pop     hl                              ; clear RET address
        ld      hl,strPongP1Won
        or      a
        jr      z,pbGameOverDisp
        ld      b,a
        ld      a,(coop)
        or      a
        ld      a,b
        ld      hl,strPongP2Won
        jr      nz,pbGameOverDisp
        ld      hl,strPongCalcWon
#ifdef TI83PLUS
pbGameOverDisp:
        push    af                              ; save won/lost flag
        call    showStrings
        call    pongShowScore
        call    xorPongSprites
        bcall(_grbufcpy)
#endif
#ifdef TI84PLUSCE
pbGameOverDisp:
        push    af
        push    hl
        call    drawPongField
        pop     ix
        call    printBlock
        call    vramFlip
#endif
        call    wait2ndEnter                    ; wait for [2nd] or [Enter]
        pop     af
        or      a                               ; check again if player 1 won
        ret     nz                              ; if not, leave
        ; check if co-op or not
        ld      a,(coop)
        or      a
        ret     nz                              ; if co-op game, can't get the achievement
        ; otherwise make sure the achievement is set
        ld      bc,ACH_WINPONG_BIT*256+ACH_WINPONG_ID
        ld      de,ACH_WINPONG_OFFSET
        ld      hl,viewNewAchievement
        bcall(ab5_setAchievement)
        ; check for flawless victory
        ld      a,(pong2score)
        or      a
        ret     nz
        ld      bc,ACH_PONGCORE_BIT*256+ACH_PONGCORE_ID
        ld      de,ACH_PONGCORE_OFFSET
        ld      hl,viewNewAchievement
        bcall(ab5_setAchievement)
        ret

#ifdef TI83PLUS
;------------------------------------------------
; pongShowScore - update the score to gbuf
;   input:  none
;   output: none
;------------------------------------------------
pongShowScore:
        ld      hl,(pong1score)
        ld      h,0
        ld      de,57*256+0
        call    setShowHL
        ld      hl,(pong2score)
        ld      h,0
        ld      de,57*256+61
        jp      setShowHL

;------------------------------------------------
; xorPongSprites - XOR all pong sprites
;   input:  none
;   output: none
;------------------------------------------------
xorPongSprites:
        ; draw ball
        ld      hl,sprPongBall
        call    spriteToRam
        ld      hl,(pongBally)                  ; L = y coord
        ld      a,h                             ; A = x coord
        call    xorPongSprite                   ; draw sprite
        ; draw paddles
        ld      hl,sprPongPaddle
        call    spriteToRam
        call    xorCalcPaddle
; fall through to xorPlayerPaddle

;------------------------------------------------
; xorPlayerPaddle - XOR player paddle to gbuf
;   input:  none
;   output: none
;------------------------------------------------
xorPlayerPaddle:
        xor     a
        ld      hl,(pong1)
        ld      h,a
        jr      xorPongSprite

;------------------------------------------------
; xorCalcPaddle - XOR calc paddle to gbuf
;   input:  none
;   output: none
;------------------------------------------------
xorCalcPaddle:
        ld      a,96-8
        ld      hl,(pong2)
        ld      h,0
; fall through to xorPongSprite

;------------------------------------------------
; xorPongSprite - XOR a sprite to gbuf
;   input:  A = x coord
;           L = y coord
;   output: none
;------------------------------------------------
xorPongSprite:
        ld      de,sprBuffer
        bcall(ab5_putSprite)
        ret
#endif

#ifdef TI83PLUS
;------------------------------------------------
; drawPongField - draw the play field (83P version)
;   input:  none
;   output: none
;------------------------------------------------
drawPongField:
        bcall(_grbufclr)
        ld      hl,gbuf
        call    pongHline
        ld      hl,gbuf+648
        call    pongHline
        ld      hl,gbuf
        ld      c,%11000000
        call    pongVline
        ld      hl,gbuf+11
        ld      c,%00000011
        call    pongVline
        ld      hl,gbuf+5
        ld      c,%00000001
        call    pongVline
        ld      hl,gbuf+6
        ld      c,%10000000
        ; fall thru to pongVline

;------------------------------------------------
; pongVline - draw vertical line 56 pixels down for play field
;   input:  HL => first byte to write to
;           C = pixel data (to be OR'd to gbuf)
;   output: none
;------------------------------------------------
pongVline:
        ld      b,56
        ld      de,12
pvlLoop:
        ld      a,(hl)
        or      c
        ld      (hl),a                          ; load new gbuf byte
        add     hl,de
        djnz    pvlLoop
        ret

;------------------------------------------------
; pongHline - draw 2 pixel thick horizontal line for play field
;   input:  HL => start of gbuf row
;   output: none
;------------------------------------------------
pongHline:
        push    hl
        pop     de
        inc     de                              ; DE => byte after (HL)
        ld      bc,23
        ld      (hl),$FF
        ldir
        ret
#endif
#ifdef TI84PLUSCE
;------------------------------------------------
; drawPongField - draw the play field, sprites & score (CE version)
;   input:  none
;   output: none
;------------------------------------------------
drawPongField:
        ; clear the portion of vbuf we'll be drawing to
        RECT_POS(42,50)
        RECT_SIZE(236,132)
        ld      a,COLOUR_BLACK
        call    fillRect
        ; draw the field
        ld      de,50*256+32
        ld      bc,95*256+112
        ld      a,COLOUR_WHITE
        call    drawRect
        ld      de,50*256+80
        ld      c,112
        ld      a,COLOUR_WHITE
        call    drawVLine
        ; draw the sprites
        ld      a,(pong1)
        add     a,a
        add     a,50
        ld      d,a
        ld      e,35
        ld      c,16
        ld      a,COLOUR_WHITE
        call    drawVLine
        ld      a,(pong2)
        add     a,a
        add     a,50
        ld      d,a
        ld      e,96-4+32
        ld      c,16
        ld      a,COLOUR_WHITE
        call    drawVLine
        ld      a,(pongBally)
        add     a,a
        add     a,50
        ld      d,a
        ld      a,(pongBallx)
        add     a,32
        ld      e,a
        ld      c,2
        ld      a,COLOUR_WHITE
        call    drawVLine
        ; draw the score
        ld      a,(pong1score)
        LDHLA()
        ld      de,90
        ld      c,170
        call    showHL
        ld      a,(pong2score)
        LDHLA()
        ld      de,184
        ld      c,170
        jp      showHL
#endif

.end
