;===============================================================;
;                                                               ;
; Alien Breed 5                                                 ;
; Text routines (83P only)                                      ;
;                                                               ;
;===============================================================;

;------------------------------------------------
; setShowHL - show HL in small font at specified position (right aligned)
;   input:  HL = value to show
;           DE = penrow, pencol
;   output: none
;------------------------------------------------
setShowHL:
        ld      (pencol),de

;------------------------------------------------
; showHL - show HL in small font at cursor position (right aligned)
;   input:  HL = value to show
;   output: none
;------------------------------------------------
showHL:
        ld      de,showHLBuffer+5               ; DE => end of string buffer
        ld      b,5                             ; B = number of characters to calculate
        xor     a
        ld      (de),a                          ; set NULL terminator at end of string buffer and work backwards from there
showHLMake:
        dec     de                              ; DE => next character back
        bcall(_divHLby10)                       ; A = number to display
        add     a,'0'                           ; A = character to display
        ld      (de),a                          ; store it
        djnz    showHLMake
        ex      de,hl                           ; HL => start of string
        ; now check string and eliminate all leading zeros, and adjust pencol accordingly (string is right-aligned)
        ld      b,4                             ; B = number of characters to check (last character is always shown)
        ld      a,(pencol)                      ; A = pencol
showHLCheck:
        ld      c,a                             ; C = pencol
        ld      a,(hl)                          ; A = char
        cp      '0'                             ; is it a 0?
        ld      a,c                             ; A = pencol
        jr      nz,showHLReady                  ; if not, ready to show string
        add     a,4                             ; A = new pencol (skipping where this character would have been shown)
        inc     hl                              ; HL => next char in string
        djnz    showHLCheck
showHLReady:
        ld      (pencol),a                      ; set new pencol
        bcall(_vputs)
        ret

;------------------------------------------------
; showStrings - show a set of strings
;   input:  HL => string data
;   output: HL => after string data
;------------------------------------------------
showStrings:
        ld      b,(hl)                          ; B = number of strings to display
        inc     hl
showStringsLoop:
        push    bc
        ld      d,(hl)                          ; D = penrow value
        inc     hl
        ld      e,(hl)                          ; E = pencol value
        inc     hl                              ; HL => string
        call    vPuts
        pop     bc
        djnz    showStringsLoop
        ret

;------------------------------------------------
; vPuts - same as _vputs but sets cursor position
;   input:  DE = value to store to (pencol)
;           HL => string to display
;   output: HL => next string
;------------------------------------------------
vPuts:
        ld      (pencol),de
vPutsNoCursor:
        ld      de,strBuffer                    ; DE => where to write it to
vPutsCopy:
        ld      a,(hl)                          ; A = character
        ld      (de),a                          ; save it
        inc     hl
        inc     de
        or      a                               ; check if NULL terminator
        jr      z,vPutsReady                    ; if so, we're ready to print
        cp      $FF                             ; check if $FF
        jr      nz,vPutsCopy                    ; if not, keep copying
        ; otherwise, go back and write over the $FF with a NULL terminator (used for displayPageFast in intex.asm)
        dec     de
        xor     a
        ld      (de),a                          ; write NULL terminator
vPutsReady:
        push    hl                              ; preserve pointer to next string
        ld      hl,strBuffer                    ; HL => string to display
        bcall(_vputs)
        pop     hl                              ; HL => next string
        ret

;------------------------------------------------
; skipString - find new string
;   input:  HL => current string (NULL terminated)
;   output: HL => next string
;------------------------------------------------
skipString:
        ld      a,(hl)
        inc     hl
        or      a
        jr      nz,skipString
        ret

.end
